#include <conio.h>
#include <stdlib.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 16

   Description:  Demonstrates mesh level of detail
*/


#define MAX_LOD 4
typedef struct {
PR_OBJECT *LODmesh[MAX_LOD];     /* A shape for each level */
} LOD_OBJECT;

typedef struct {
LOD_OBJECT *shapelist;
PR_ENTITY *entity;
} LOD_ENTITY;


LOD_OBJECT myobj;
LOD_ENTITY myent;

PR_DWORD last_lod = 0;

PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_LIGHTLIST userlights;               /* Holds a list of lights */


void LoadObjects (void)
{
  myobj.LODmesh[0] = PR_LoadPRO ("ball0.pro", LOAD_NORMAL);
  myobj.LODmesh[1] = PR_LoadPRO ("ball1.pro", LOAD_NORMAL);
  myobj.LODmesh[2] = PR_LoadPRO ("ball2.pro", LOAD_NORMAL);
  myobj.LODmesh[3] = PR_LoadPRO ("ball3.pro", LOAD_NORMAL);
}


PR_DWORD DetermineLOD (PR_REAL dist)
{
  if (dist < 1000)
    return 0;
  else if (dist < 5000)
    return 1;
  else if (dist < 15000)
    return 2;
  else
    return 3;
}


void DrawLODEntity (LOD_ENTITY *ent)
{
PR_REAL dist;
PR_DWORD lod;

  dist = PR_Distance3D (camera->source.x, camera->source.y, camera->source.z,
                        ent->entity->orientation.location.x,
                        ent->entity->orientation.location.y,
                        ent->entity->orientation.location.z);
  lod = DetermineLOD (dist);
  last_lod = lod;

  ent->entity->shape = ent->shapelist->LODmesh[lod];

  PR_TransformEntity (ent->entity);
  PR_RenderEntity (ent->entity);
}




void main (int argc, char *argv[])
{
PR_DWORD vwidth, vheight;
PR_UDWORD BLACK;                       /* Color index/value for black */

  ChooseDevice ();

  PR_Initialize (4000);                /* Maximum 4000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */


  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  PR_SetMode (vwidth, vheight, 60);

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  LoadObjects ();

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  myent.entity = PR_CreateEntity (myobj.LODmesh[0], "TEST ENTITY");
  myent.shapelist = &myobj;

  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 4000);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, 4000);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */


  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

  PRGFX_SetTextForeground (PRGFX_MakeColor (63, 63, 63));
  PRGFX_SetTextTransparent (TEXTFG);

  installkbd ();

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();                   /* Begin a new frame */

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     PR_TransformLights (&userlights);

     if (kbdon[KEY_UP])
       PR_MoveEntity (myent.entity, 0, 0, 50);

     if (kbdon[KEY_DOWN])
       PR_MoveEntity (myent.entity, 0, 0, -50);

     DrawLODEntity (&myent);

     PR_RenderFrame ();                /* Draw the scene */

     PRGUI_printf (0, 0, "LOD: %i", last_lod);
     PRGUI_printf (0, 10, "Triangles Drawn: %i", PR_StatElementsDrawn ());

     PR_Flip (1);                      /* Show the background page */
    }

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (myent.entity);
  PR_FreeObject (myobj.LODmesh[0]);
  PR_FreeObject (myobj.LODmesh[1]);
  PR_FreeObject (myobj.LODmesh[2]);
  PR_FreeObject (myobj.LODmesh[3]);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();

  uninstallkbd ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

