#include <conio.h>
#include <stdlib.h>
#include <math.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 17

   Description:  Demonstrates projecting points and perspective scaling
   

   Controls:
   Click and drag the left mouse button to move the object along the
   x/y axis.

   Click and drag the right mouse button to move the object along the
   z axis.

   Press ESC to quit.
*/

                                             
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_LIGHTLIST userlights;               /* Holds a list of lights */

PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *ball;

PR_DWORD vwidth, vheight;

PR_UDWORD BLACK;                       /* Color index/value for black */
PR_UDWORD WHITE;                       /* Color index/value for white */


void DrawBoundingRectangle (PR_ENTITY *ent)
{
PR_VERTEX vert;
PR_VERTEX_DATA vdata;

PR_VERTEX vert2;
PR_VERTEX_DATA vdata2;

PR_VERTEX vert3;
PR_VERTEX_DATA vdata3;

PR_REAL width, height;
PR_REAL sx, sy;

  PRGFX_Clip (0, 0, vwidth-1, vheight-1);

  /* Set up a vertex to match the center of the entity */
  vert.x = ent->orientation.location.x;
  vert.y = ent->orientation.location.y;
  vert.z = ent->orientation.location.z;
  vert.vdata = &vdata;

  /* Set up a vertex to match the side of the entity */
  vert2.x = ent->orientation.location.x + ent->shape->bbox.radius;
  vert2.y = ent->orientation.location.y;
  vert2.z = ent->orientation.location.z;
  vert2.vdata = &vdata2;

  /* Set up a vertex to match the top of the entity */
  vert3.x = ent->orientation.location.x;
  vert3.y = ent->orientation.location.y + ent->shape->bbox.radius;
  vert3.z = ent->orientation.location.z;
  vert3.vdata = &vdata3;

  /* Transform the vertices into view space */
  PR_TransformVertexList (&vert, &vdata, 1, PR_ViewMatrix);
  PR_TransformVertexList (&vert2, &vdata2, 1, PR_ViewMatrix);
  PR_TransformVertexList (&vert3, &vdata3, 1, PR_ViewMatrix);

  /* Project to screen coordinates.  We do this ourselves because the
     PR_TransformVertexList only projects if the vertex is within
     the viewing volume. */

  if (vdata.vz > 0)
    {
     /* Do the projection to screen space */
     sx = (vdata.vx * vdata.ooz * active_viewport.xscale) + active_viewport.centerx;
     sy = active_viewport.centery - (vdata.vy * vdata.ooz * active_viewport.yscale);

     width = abs((vdata2.vx * vdata2.ooz * active_viewport.xscale) + active_viewport.centerx - sx);
     height = abs(active_viewport.centery - (vdata3.vy * vdata3.ooz * active_viewport.yscale) - sy);

     PRGFX_SetColor (WHITE);
     PRGFX_Rectangle (sx - width, sy - height, sx + width, sy + height);
    }
}



void main (void)
{
PR_REAL cenx, ceny, cenz;


  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */


  vwidth = 640;
  vheight = 480;
  if (PR_SetMode (vwidth, vheight, 60) != MODE_SUCCESS)
    {
     vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
     vheight = PR_VideoModes[0].height;
     PR_SetMode (vwidth, vheight, 60);
    }

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  /* Load the object */
  PR_SetTexturePath ("..\\data\\");
  PR_SetTablePath ("..\\data\\");
  object_shape = PR_LoadPRO ("..\\data\\sphere.pro", LOAD_NORMAL);
  PR_CenterObject (object_shape, &cenx, &ceny, &cenz);

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  ball = PR_CreateEntity (object_shape, "Ball 1");

  PR_PositionEntity (ball, 0, 0, 0);

  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, -900);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, -900);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);
  WHITE = PRGFX_MakeColor (63, 63, 63);

  installkbd ();

#ifndef WIN32
  minit ();
#endif

  msetbounds (0, 0, 320, 200);
  msetxy (160, 100);

  /* Load a smaller font */
  gui_font = wloadfont ("little.wfn");
  PRGFX_SetTextForeground (WHITE);
  PRGFX_SetTextBackground (BLACK);
  PRGFX_SetTextTransparent (TEXTFGBG);

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
          UpdateMessages ();
         #endif

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();                   /* Begin a new frame */

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     PR_TransformLights (&userlights);


     if (mouse.but == 1)
       {
        PR_MoveEntity (ball, mouse.mx - 160, 100 - mouse.my, 0);
        msetxy (160, 100);
       }
     else if (mouse.but == 2)
       {
        PR_MoveEntity (ball, 0, 0, 100 - mouse.my);
        msetxy (160, 100);
       }


     PR_TransformEntity (ball);       /* Transform the entity */
     PR_RenderEntity (ball);          /* Render the entity */

     PR_RenderFrame ();                /* Draw the scene */

     DrawBoundingRectangle (ball);

     PR_Flip (1);                      /* Show the background page */
    }

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (ball);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();

  uninstallkbd ();
  mdeinit ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

