#include <conio.h>
#include <stdlib.h>
#include <math.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 18

   Description:  Demonstrates triangle picking
   
   Controls:
   Click the left/right buttons on a triangle to change the material it uses.

   Press ESC to quit.

*/

                                             
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_LIGHTLIST userlights;               /* Holds a list of lights */

PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *ball;

PR_DWORD vwidth, vheight;

PR_UDWORD BLACK;                       /* Color index/value for black */
PR_UDWORD WHITE;                       /* Color index/value for white */

PR_DWORD picked_material, notpicked_material;
PR_DWORD segval;
PR_RAY pickray;


void MakeRay (PR_DWORD x, PR_DWORD y)
/* Converts a screen coordinate into a ray from the origin */
{
PR_REAL z_times_ooxscale;
PR_REAL z_times_ooyscale;

PR_POINT start, end;
PR_DWORD res;

  start.x = 0;
  start.y = 0;
  start.z = 0;

  /* Undo the perspective transform.... turning a screen coordinate back
     into a 3D point with Z=65535. */
  end.z = 65535.0f;
  z_times_ooxscale = end.z * (1.0f / active_viewport.xscale);
  z_times_ooyscale = end.z * (1.0f / active_viewport.yscale);

  end.x = (x - active_viewport.centerx) * z_times_ooxscale;
  end.y = (active_viewport.centery - y) * z_times_ooyscale;

  res = PRCL_RayFromPoints (&pickray, &start, &end);
  if (res == PRCL_FALSE)
    PR_FatalError ("Invalid ray", "Ex18");
}


PR_DWORD PickTriangle (PR_OBJECT *obj, PR_DWORD *segval)
/* The basic idea here is to find which intersecting triangle if any has the
   closest z coordinate.  The segment number of the face is stored in the
   segval parameter.

   If no face is found, it returns -1.  
*/
{
PR_REAL minz, tri_minz;
PR_SEGMENT *seg;
PR_FACE *face;
PR_DWORD segnum, facenum;
PR_DWORD pickedface, res;
PR_REAL t;

  pickedface = -1;
  minz = 65535.0f;

  seg = obj->segment_list;
  for (segnum = 0; segnum < obj->num_segments; segnum++)
    {
     face = seg->face_list;
     for (facenum = 0; facenum < seg->num_faces; facenum++)
       {
        res = PRCL_RayIntersectsTriangle (&pickray, face->vertex1,
                        face->vertex2, face->vertex3, &t);
        if (res & PRCL_TRUE)
        /* The ray intersects this face */
          {
           /* Find the vertex with the closest z */
           tri_minz = face->vertex1->vdata->vz;
           if (face->vertex2->vdata->vz < tri_minz)
             tri_minz = face->vertex2->vdata->vz;
           if (face->vertex3->vdata->vz < tri_minz)
             tri_minz = face->vertex2->vdata->vz;

           /* If this face is closer than what we found so far */
           if ((tri_minz > 0) && (tri_minz < minz))
             {
              minz = tri_minz;
              pickedface = facenum;
              *segval = segnum;
             }
          }
        face++;
       }
     seg++;
    }

  return pickedface;
}


void SwitchMaterial (PR_OBJECT *obj, PR_DWORD segnum, PR_DWORD facenum,
                    PR_DWORD matnum)
{
  obj->segment_list[segnum].face_list[facenum].material = matnum;
}



void main (void)
{
PR_REAL cenx, ceny, cenz;
PR_DWORD facenum;

  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */


  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  if (PR_SetMode (vwidth, vheight, 60) != MODE_SUCCESS)
    {
     vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
     vheight = PR_VideoModes[0].height;
     PR_SetMode (vwidth, vheight, 60);
    }

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  /* Load the object */
  object_shape = PR_LoadPRO ("blob.pro", LOAD_NORMAL);
  PR_CenterObject (object_shape, &cenx, &ceny, &cenz);

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  ball = PR_CreateEntity (object_shape, "Ball 1");
  PR_PositionEntity (ball, 0, 0, 0);

  picked_material = PR_FindMaterial ("picked");
  notpicked_material = PR_FindMaterial ("notpicked");


  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, -1400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, -1400);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);
  WHITE = PRGFX_MakeColor (63, 63, 63);

  installkbd ();

#ifndef WIN32
  minit ();
#endif

  msetbounds (0, 0, vwidth-1, vheight-1);
  msetxy (vwidth/2, vheight/2);

  /* Load a smaller font */
  gui_font = wloadfont ("little.wfn");
  PRGFX_SetTextForeground (WHITE);
  PRGFX_SetTextBackground (BLACK);
  PRGFX_SetTextTransparent (TEXTFGBG);

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
          UpdateMessages ();
         #endif

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();                   /* Begin a new frame */

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     PR_TransformLights (&userlights);

     PR_RotateEntity (ball, 0.1, 0.12, 0.14);
     PR_TransformEntity (ball);       /* Transform the entity */


     if (mouse.but == 1)
       {
        MakeRay (mouse.mx, mouse.my);
        facenum = PickTriangle (object_shape, &segval);
        if (facenum >= 0)
          SwitchMaterial (object_shape, segval, facenum, picked_material);
       }
     else if (mouse.but == 2)
       {
        MakeRay (mouse.mx, mouse.my);
        facenum = PickTriangle (object_shape, &segval);
        if (facenum >= 0)
          SwitchMaterial (object_shape, segval, facenum, notpicked_material);
       }

     PR_RenderEntity (ball);          /* Render the entity */

     PR_RenderFrame ();                /* Draw the scene */

     PRGFX_SetColor (WHITE);
     PRGFX_Line (mouse.mx - 10, mouse.my, mouse.mx + 10, mouse.my);
     PRGFX_Line (mouse.mx, mouse.my - 10, mouse.mx, mouse.my + 10);

     PR_Flip (1);                      /* Show the background page */
    }

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (ball);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();

  uninstallkbd ();
  mdeinit ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

