#include <conio.h>
#include <stdlib.h>
#include <math.h>

#include <pr.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prgui.h>
#include <prdev.h>

/* Power Render Example 19

   Description:  Shows how to render a single triangle given its coordinates in world space

*/

                                             
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_LIGHTLIST userlights;               /* Holds a list of lights */

PR_MATERIAL mat;
PR_DWORD mymat, mytex;

PR_VERTEX vlist[3];
PR_VERTEX_DATA vlistdata[3];
PR_FACE face;



void CreateMaterial (void)
{
  mytex = PR_LoadTexture ("box.pcx");

  PR_SetMaterialName (&mat, "mymat");
  PR_SetMaterialMethod (&mat, T_PTEXTURED);
  PR_SetMaterialBaseColor (&mat, 0);
  PR_SetMaterialColor (&mat, 255, 255, 255);
  PR_SetMaterialTexture (&mat, mytex);
  PR_SetMaterialTable (&mat, 0);
  PR_SetMaterialShades (&mat, 31);
  PR_SetMaterialEnvironmentMap (&mat, 0);
  PR_SetMaterialMipMapState (&mat, FALSE);
  PR_SetMaterialAlpha (&mat, 255);
  mymat = PR_AddMaterial (&mat);
}


void CreateTriangle (void)
{
PR_FACE_DATA_REAL *fdatareal;
PR_FACE_DATA *fdata;

  /* First create the vertices in world space */
  vlist[0].x = 0;
  vlist[0].y = -100;
  vlist[0].z = 1;
  vlist[0].vdata = &vlistdata[0];

  vlist[1].x = 150;
  vlist[1].y = 50;
  vlist[1].z = 1;
  vlist[1].vdata = &vlistdata[1];

  vlist[2].x = -150;
  vlist[2].y = 50;
  vlist[2].z = 1;
  vlist[2].vdata = &vlistdata[2];

  /* If you are using a shaded rendering method, you'll also need to
     initialize the vertex and face normals */

  /* Now create the face structure */
  face.vertex1 = &vlist[0];
  face.vertex2 = &vlist[1];
  face.vertex3 = &vlist[2];
  face.material = mymat;
  face.flags = FFLAG_FRONT_VISIBLE | FFLAG_BACK_VISIBLE;

  /* Set the texture mapping coordinates.  If using 3D hardware, they
     will be floating point numbers, and the range depends on the
     underlying API (Glide, Direct3D, or OpenGL).  If software rendering
     is used, they are in 16.16 fixed point format.
     To convert to the proper scale, use the range 0-255 and multiply
     by the scale of the texture.  */
  if (PR_Settings.RealUV)
    {
     /* Cast it to floating point form */
     fdatareal = (PR_FACE_DATA_REAL *)&face.face_data;
     fdatareal->u[0] = 0;
     fdatareal->v[0] = 0;

     fdatareal->u[1] = 255 * PR_WorldTextures[mytex].xscale;
     fdatareal->v[1] = 0;

     fdatareal->u[2] = 255 * PR_WorldTextures[mytex].xscale;
     fdatareal->v[2] = 255 * PR_WorldTextures[mytex].yscale;
    }
  else
    {
     /* Software mode... The example texture is 128x128 so the maximum
        texture coordinate is 127<<16 */
     fdata = &face.face_data;

     fdata->u[0] = 1;
     fdata->v[0] = 1;

     fdata->u[1] = 127 << 16;
     fdata->v[1] = 1;

     fdata->u[2] = 127 << 16;
     fdata->v[2] = 127 << 16; 
    }
}


void DrawTriangle (void)
{
  PR_TransformVertexList (vlist, vlistdata, 3, PR_ViewMatrix);
  PR_AddElement (&face, vlist, vlistdata, 1);
}



void main (int argc, char *argv[])
{
PR_DWORD vwidth, vheight;
PR_UDWORD BLACK;                       /* Color index/value for black */
PR_REAL campos = 0.0f;

  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */
  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */


  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  if (PR_SetMode (vwidth, vheight, 60) != MODE_SUCCESS)
    PR_FatalError ("Cannot set video mode", NULL);


  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  CreateMaterial ();
  CreateTriangle ();
 
#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, 400);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

  installkbd ();

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  PR_SetPerspectiveDivisions (TEXTURE_DIVISION_8);
#endif

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif      

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();                   /* Begin a new frame */

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     campos += 0.015f;
     PR_PositionCameraSource (camera, cos (campos)*400, 0, sin(campos)*400);
     PR_SetActiveCamera (camera);

     PR_TransformLights (&userlights);

     DrawTriangle ();

     PR_RenderFrame ();                /* Draw the scene */

     PR_Flip (1);                      /* Show the background page */
    }

  uninstallkbd ();

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

