#ifndef _GAME_
#define _GAME_

#ifdef __cplusplus
  extern "C" {
#endif

#define CAMERA_DISTANCE -300

#define FOG_START 320
#define FOG_END 420

#define EXPLOSION_LIGHT_SIZE 2048
#define WEAPON_LIGHT_SIZE 2048

#define WORLD_SCALE 512
#define HEIGHT_SCALE 64

#define ABOVE_GROUND 125
#define MAX_HEIGHT 16000

#define NUM_TILES 256
#define OBJ_TILES 256           /* Number of materials allowed for objects */



/* 3D Engine */
extern PR_CAMERA *   camera;
extern PR_VIEWPORT   viewport;                  /* Our viewport structure */
extern PR_DWORD      vwidth, vheight;           /* Viewport width and height */
extern PR_REAL       camera_fov;

/* 3D Objects */
extern PR_OBJECT *sky_shape;
extern PR_ENTITY *sky_entity;
extern PR_OBJECT *spaceship_shape;
extern PR_ENTITY *spaceship_entity;


/* Terrain */
extern PR_TERRAIN *terrain;
extern PR_DWORD terrain_size;


/* Textures and bitmaps */
extern block tilesprites[1024];


/* Palettes and Colors */
extern color temppal[256];

/* Lights */
extern PR_LIGHTLIST userlights;
extern PR_LIGHTLIST explosionlights;
extern PR_LIGHTLIST sunlight;
extern PR_LIGHTLIST weaponlight;

/* User Input */
extern PR_REAL spinspeed;
extern PR_REAL camera_tilt;
extern PR_MATRIX rollmatrix;
extern PR_REAL TILT_SPEED;
extern PR_REAL MAX_TILT_SPEED;
extern PR_REAL TILT_DRAG;


/* Frame Rate Control  */
extern PR_DWORD animation_count;   /* ticks since last draw */
extern PR_DWORD ticks;             /* Total number of ticks passed for fps */
extern PR_DWORD updates;           /* Total number of updates */
extern PR_DWORD framerate;         /* Resulting frame rate (updates/ticks/tickrate) */


void EntityCoordinateFix (PR_ENTITY *ent);
void AddExplosion (PR_REAL x, PR_REAL y, PR_REAL z);

/* Weapons */
extern PR_DWORD shoot_wing;

#define MAX_PARTICLE_SHAPES 20
extern PR_OBJECT *ParticleObjects[MAX_PARTICLE_SHAPES];
extern PR_ENTITY *ParticleEntities[MAX_PARTICLE_SHAPES];
extern PR_DWORD   ParticleMaterials[MAX_PARTICLE_SHAPES];

/* Emitter Templates */
#define FLARETRAIL_EMITTER 0
#define SMOKE_EMITTER      1
#define ASH_EMITTER        2
#define FIRE_EMITTER       3
#define SPARK_EMITTER      4
#define BOUNCEPUFF_EMITTER 5
#define EXPLOSION1_EMITTER 6
#define EXPLOSION2_EMITTER 7

/* Particle Templates */
#define FLARETRAIL_PARTICLE 0
#define SMOKE_PARTICLE      1
#define ASH_PARTICLE        2
#define FIRE_PARTICLE       3
#define SPARK_PARTICLE      4
#define BOUNCEPUFF_PARTICLE 5
#define EXPLOSION1_PARTICLE 6
#define EXPLOSION2_PARTICLE 7

void InitializeWeapons (void);
void FireWeapon1 (PR_ENTITY *entity);


/* Enemies */
#define MAX_ENEMY_SHAPES 10
#define MAX_ENEMY_ENTITIES 50
void LoadEnemyData (void);
void InitializeEnemies (void);
void UpdateEnemies (PR_DWORD ticks_passed);


/* Sound Effects */
#define MAXWAVE 15
#define WAVE_WEAPON1  0
#define WAVE_HITLAND  1
#define WAVE_EXPLODE1 2
#define WAVE_SCRAPE   3
#define WAVE_PROBE    4
#define WAVE_TANK     5

extern PR_DWORD Waves[MAXWAVE];

extern PR_3DSOUND *sound_weap1;
extern PR_3DSOUND *sound_hitland;
extern PR_3DSOUND *sound_explode1;
extern PR_3DSOUND *sound_scrape;
extern PR_3DSOUND *sound_enemy[MAX_ENEMY_ENTITIES];
void InitializeAudio (void);
void CloseAudio (void);


#ifdef __cplusplus
}
#endif

#endif

