#include <stdlib.h>
#include <direct.h>

#include <pr.h>
#include <prgui.h>
#include <prsound.h>


#if !defined (WIN32)
  #include <terrain.h>
  #include <pr3dfx.h>
  #include <glide.h>
#else
  #if defined (MSGLIDE) || defined (WTGLIDE)
    #include <pr3dfx.h>
    #include <glide.h>
  #endif
  #include <terrain.h>
  #include <winutil.h>
#endif

#include "game.h"


/* ---------------------------------------------------------------------- */
/* Timer Interrupt */
/* ---------------------------------------------------------------------- */
void timerproc (void)
{
  animation_count++;
  ticks++;
  AUpdateAudio ();
}                



void InitializeInputDevices (void)
{
#ifndef WIN32
  vga256 ();
  minit ();
#endif

  msetbounds (0, 0, 319, 199);
  msetxy (160, 100);
  /* Necessary for the standard input routines */

  installkbd ();

  winittimer ();
  wstarttimer (timerproc, TICKS(30));
}


void Initialize3DEngine (void)
{
PR_DWORD i;

  PR_AllocMaterials (256 * 6);
  PR_AllocTextures (256 * 6);
  PR_AllocShadeTables (32);

  vwidth = 640;
  vheight = 480;
  PR_SetMode (vwidth, vheight, 60);

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  PR_Settings.HardwareMipmaps = 1;

  gui_font = NULL;
  PRGFX_SetTextForeground (PRGFX_MakeColor (63,63,63));
  PRGFX_SetTextBackground (PRGFX_MakeColor (32,32,32));
  PRGFX_SetTextTransparent (TEXTFGBG);

  PR_AmbientLight = 0;
  PR_AmbientRed = 0;
  PR_AmbientGreen = 0;
  PR_AmbientBlue = 0;

  PR_SetFogRange (terrain_size * FOG_START, terrain_size * FOG_END);
  PR_SetFogState (FALSE);

  camera = PR_AllocCamera ();
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 2400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  camera->fov = camera_fov / 3.0f * 3.1415f;

  PR_MatrixRotate (rollmatrix, 0, 0, 0);

  PR_AllocLights (&weaponlight, 1);      /* Weapon light */
  PR_AllocLights (&userlights, 60);      /* User lights */
  PR_AllocLights (&sunlight, 1);         /* Infinite sun light */
  PR_AllocLights (&scenelights, 100);     /* Master scene light list */
  PR_AllocLights (&explosionlights, 60);
  for (i = 0; i < 20; i++)
    {
     PR_SetLightOff (&explosionlights, i);
     PR_SetLightType (&explosionlights, i, POINT_LIGHT);
     PR_SetLightFalloff (&explosionlights, i, EXPLOSION_LIGHT_SIZE);
     PR_SetLightColor (&explosionlights, i, 1.0, 0.5, 0.0);
    }
  explosionlights.NumLights = 20;

  PR_SetLightOn (&weaponlight, 0);
  PR_SetLightType (&weaponlight, 0, POINT_LIGHT);
  PR_SetLightFalloff (&weaponlight, 0, WEAPON_LIGHT_SIZE);
  PR_SetLightColor (&weaponlight, 0, 0.0, 0.0, 1.0);
  weaponlight.NumLights = 1;     

  PR_SetLightPosition (&sunlight, 0,170000, 170000, 170000);
  PR_SetLightType (&sunlight, 0, DIRECTIONAL_LIGHT);
  PR_SetLightOn (&sunlight, 0);
  PR_SetLightStrength (&sunlight, 0, 1.0);
  PR_SetLightColor (&sunlight, 0, 0.5, 0.5, 0.5);

  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  PR_SetLightFalloff (&userlights, 0, 768);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);

  userlights.NumLights = 0;     /* This sets how many lights are actually used */
  sunlight.NumLights = 1;     
}


void LoadGameData (void)
{
  /* -------------- Load objects ----------------- */
  terrain = PR_AllocateTerrain (terrain_size,
                                TERRAIN_PLAIN | TERRAIN_SHADING,
                                WORLD_SCALE, HEIGHT_SCALE, OBJ_TILES);
  /* Load in the height and tile maps */
  PR_LoadTerrain (terrain, 1, "hf1.pcx", "track1.wmp", "shading.pcx", 32);
//  PR_LoadTerrain (terrain, 2, "hf2.pcx", "track2.wmp", "shading.pcx", 32);
//  PR_LoadTerrain (terrain, 3, "hf3.pcx", "track3.wmp", "shading.pcx", 32);

  spaceship_shape = PR_LoadPRO ("ship.pro", LOAD_NORMAL);
  spaceship_entity = PR_CreateEntity (spaceship_shape, "You");

  PR_SetTextureFormat (TEXTURE_P_8);
  sky_shape = PR_LoadPRO ("cube.pro", LOAD_NORMAL);
  PR_SetTextureFormat (TEXTURE_NORMAL);
  sky_entity = PR_CreateEntity (sky_shape, "Sky");
  PR_PositionEntity (sky_entity, 0, 0, 0);
  PR_ScaleEntityAbs (sky_entity, 160, 160, 160);
}

