#include <stdlib.h>
#include <direct.h>

#include <pr.h>
#include <prgui.h>
#include <prsound.h>


#if !defined (WIN32)
  #include <terrain.h>
  #include <pr3dfx.h>
  #include <glide.h>
#else
  #if defined (MSGLIDE) || defined (WTGLIDE)
    #include <pr3dfx.h>
    #include <glide.h>
  #endif
  #include <terrain.h>
  #include <winutil.h>
#endif

#include "game.h"


#define MAXWEAPON 10
PR_POINT WeaponOffsets[MAXWEAPON] = {
{-110, -20, 50},
{110, -20, 50}
};

PR_POINT WeaponVectors[MAXWEAPON] = {
{0, 0, 200},
{0, 0, 200}
};

PR_OBJECT *ParticleObjects[MAX_PARTICLE_SHAPES];
PR_ENTITY *ParticleEntities[MAX_PARTICLE_SHAPES];
PR_DWORD   ParticleMaterials[MAX_PARTICLE_SHAPES];


void DrawParticle (PR_PARTICLE *particle)
{
  EntityCoordinateFix (particle->Entity);
  PR_TransformEntity (particle->Entity);
  PR_RenderEntity (particle->Entity);
}


void Weapon1Collide (PR_EMITTER *emit)
{
PR_REAL hgt;
PR_POINT dvector, location;

  hgt = PR_GetTerrainHeight (terrain,
        emit->Location.x, emit->Location.z,
        terrain->level1_heightpic, terrain->level1_colorpic);

  if (hgt > emit->Location.y)
    {
     location.x = emit->Location.x;
     location.y = emit->Location.y;
     location.z = emit->Location.z;
     dvector.x = dvector.y = dvector.z = 0.0f;
     PR_CreateEmitter (SMOKE_EMITTER, &location, &dvector);
     PR_CreateEmitter (EXPLOSION1_EMITTER, &location, &dvector);
     AddExplosion (location.x, location.y, location.z);

     PRSND_Set3DSoundCoordinate (sound_hitland,
                                 location.x,
                                 location.y,
                                 location.z,
                                 NULL);
     PRSND_Play3DSound (sound_hitland, Waves[WAVE_HITLAND]);
     PRSND_Update3DSound (sound_hitland);
     PRSND_UpdateVoices ();

     PR_RemoveEmitter (emit->Index);
    }
  else
    {
     PR_SetLightPosition (&weaponlight, 0, emit->Location.x, emit->Location.y, emit->Location.z);
     PR_SetLightColor (&weaponlight, 0, 0.0, 0.0f, 1.0f);
     PR_TransformLights (&weaponlight);
    }
}



void BouncePuffCollide (PR_EMITTER *emit)
{
PR_REAL hgt;
PR_POINT dvector;
PR_POINT location;

  hgt = PR_GetTerrainHeight (terrain,
        emit->Location.x, emit->Location.z,
        terrain->level1_heightpic, terrain->level1_colorpic);

  if (hgt > emit->Location.y)
    {
     emit->Location.y = 0;
     emit->Velocity.y *= emit->BounceFactor.y;
     emit->Velocity.x += (rand () % 5) - 2;
     emit->Velocity.z += (rand () % 5) - 2;

     dvector.x = 0.0f;
     dvector.y = 0.0f;
     dvector.z = 0.0f;
     location.x = emit->Origin.x;
     location.y = hgt;
     location.z = emit->Origin.z;
     PR_CreateEmitter (EXPLOSION1_EMITTER, &location, &dvector);

     PRSND_Set3DSoundCoordinate (sound_hitland,
                                 location.x,
                                 location.y,
                                 location.z,
                                 NULL);
     PRSND_Play3DSound (sound_hitland, Waves[WAVE_HITLAND]);
     PRSND_Update3DSound (sound_hitland);
     PRSND_UpdateVoices ();

     emit->BounceCount--;
     if (emit->BounceCount < 1)
       PR_RemoveEmitter (emit->Index);
    }
}


void CreateParticleMaterials (void)
{
PR_MATERIAL m;
PR_DWORD i;

  PR_SetTextureFormat (TEXTURE_P_8);

  /* Load the textures used by particles */
  PR_LoadTexture ("flare1.pcx");
  PR_LoadTexture ("ash.pcx");
  PR_LoadTexture ("ash2.pcx");
  PR_LoadTexture ("smoke.pcx");
  PR_LoadTexture ("explode.pcx");

  /* Create some materials */

  /* Common values to all materials */
  PR_SetMaterialMethod (&m, T_LIGHTMAP_ADD);
  PR_SetMaterialColor (&m, 255, 255, 255);
  PR_SetMaterialShades (&m, 31);
  PR_SetMaterialEnvironmentMap (&m, 0);
  PR_SetMaterialMipMapState (&m, FALSE);
  PR_SetMaterialAlpha (&m, 255);

  PR_SetMaterialName (&m, "flare1");
  PR_SetMaterialTexture (&m, PR_FindTexture("flare1.pcx"));
  ParticleMaterials[0] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "smoke");
  PR_SetMaterialTexture (&m, PR_FindTexture("smoke.pcx"));
  ParticleMaterials[1] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "ash");
  PR_SetMaterialTexture (&m, PR_FindTexture("ash.pcx"));
  ParticleMaterials[2] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "fireball");
  PR_SetMaterialTexture (&m, PR_FindTexture("explode.pcx"));
  ParticleMaterials[3] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "ash2");
  PR_SetMaterialTexture (&m, PR_FindTexture("ash2.pcx"));
  ParticleMaterials[4] = PR_AddMaterial (&m);

  ParticleMaterials[BOUNCEPUFF_PARTICLE] = ParticleMaterials[SMOKE_PARTICLE];

  /* Now create sprite objects and entities that use these materials */
  for (i = 0; i < 6; i++)
    {
     ParticleObjects[i] = PR_AllocSprite (ParticleMaterials[i]);
     ParticleEntities[i] = PR_CreateEntity (ParticleObjects[i], "particle");

     ParticleEntities[i]->userdata[0] = ParticleMaterials[i];
     /* This is required to let the particle system know which material to
        use for alpha fading. */
    }


  ParticleObjects[EXPLOSION1_PARTICLE] = PR_LoadPRO ("explode.pro", LOAD_NORMAL);
  ParticleEntities[EXPLOSION1_PARTICLE] = PR_CreateEntity (ParticleObjects[EXPLOSION1_PARTICLE], "particle");
  ParticleEntities[EXPLOSION1_PARTICLE]->userdata[0] = PR_FindMaterial ("explode");

  ParticleObjects[EXPLOSION2_PARTICLE] = PR_LoadPRO ("ring.pro", LOAD_NORMAL);
  ParticleEntities[EXPLOSION2_PARTICLE] = PR_CreateEntity (ParticleObjects[EXPLOSION2_PARTICLE], "particle");
  ParticleEntities[EXPLOSION2_PARTICLE]->userdata[0] = PR_FindMaterial ("ring");
}


void InitializeParticleTemplates (void)
{
PR_POINT bouncefactor;


  PR_AllocParticles (2000, 20);
  PR_AllocEmitters (100, 20);

  /* Flaretrail */
  PR_InitializeParticleType (FLARETRAIL_PARTICLE);
  PR_SetParticleAlpha (FLARETRAIL_PARTICLE, 255, -30);
  PR_SetParticleScale (FLARETRAIL_PARTICLE, 1.0f, -0.063f);
  PR_SetParticleEntity (FLARETRAIL_PARTICLE, ParticleEntities[FLARETRAIL_PARTICLE]);
  PR_SetParticleDrawProc (FLARETRAIL_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (FLARETRAIL_EMITTER);
  PR_SetEmitterMovement (FLARETRAIL_EMITTER, 1);
  PR_SetEmitterTimeLimit (FLARETRAIL_EMITTER, 200);
//  PR_SetEmitterDelay (FLARETRAIL_EMITTER, 1);
  PR_SetEmitterParticleType (FLARETRAIL_EMITTER, FLARETRAIL_PARTICLE);
  PR_SetEmitterCollisionProc (FLARETRAIL_EMITTER, Weapon1Collide);

  /* Rising Smoke */
  PR_InitializeParticleType (SMOKE_PARTICLE);
  PR_SetParticleAlpha (SMOKE_PARTICLE, 255, -2);
  PR_SetParticleScale (SMOKE_PARTICLE, 0.1f, 0.013f);
  PR_SetParticleGravity (SMOKE_PARTICLE, 0.043f);  /* positive goes up */
  PR_SetParticleMovement (SMOKE_PARTICLE, 1);
  PR_SetParticleEntity (SMOKE_PARTICLE, ParticleEntities[SMOKE_PARTICLE]);
  PR_SetParticleDrawProc (SMOKE_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (SMOKE_EMITTER);
  PR_SetEmitterTimeLimit (SMOKE_EMITTER, 1);
  PR_SetEmitterParticleType (SMOKE_EMITTER, SMOKE_PARTICLE);


  /* Ash Explosion */
  PR_InitializeParticleType (ASH_PARTICLE);
  PR_SetParticleAlpha (ASH_PARTICLE, 255, -4);
  PR_SetParticleMovement (ASH_PARTICLE, 1);
  PR_SetParticleGravity (ASH_PARTICLE, -0.093f);
  PR_SetParticleScale (ASH_PARTICLE, 0.2f, 0.0f);
  PR_SetParticleEntity (ASH_PARTICLE, ParticleEntities[ASH_PARTICLE]);
  PR_SetParticleDrawProc (ASH_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (ASH_EMITTER);
  PR_SetEmitterTimeLimit (ASH_EMITTER, 1);
  PR_SetEmitterNumParticles (ASH_EMITTER, 10);
  PR_SetEmitterParticleType (ASH_EMITTER, ASH_PARTICLE);
  PR_SetEmitterRandomVelocity (ASH_EMITTER, 1);


  /* Big explosion sprite */
  PR_InitializeParticleType (FIRE_PARTICLE);
  PR_SetParticleAlpha (FIRE_PARTICLE, 255, -2);
  PR_SetParticleScale (FIRE_PARTICLE, 0.3f, 0.03f);
  PR_SetParticleGravity (FIRE_PARTICLE, 0.043f);  /* positive goes up */
  PR_SetParticleMovement (FIRE_PARTICLE, 1);
  PR_SetParticleEntity (FIRE_PARTICLE, ParticleEntities[FIRE_PARTICLE]);
  PR_SetParticleDrawProc (FIRE_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (FIRE_EMITTER);
  PR_SetEmitterTimeLimit (FIRE_EMITTER, 1);
  PR_SetEmitterParticleType (FIRE_EMITTER, FIRE_PARTICLE);

  /* Spark Explosion */
  PR_InitializeParticleType (SPARK_PARTICLE);
  PR_SetParticleAlpha (SPARK_PARTICLE, 255, -1);
  PR_SetParticleMovement (SPARK_PARTICLE, 1);
  PR_SetParticleGravity (SPARK_PARTICLE, -0.123f);
  PR_SetParticleScale (SPARK_PARTICLE, 0.6f, 0.0f);
  PR_SetParticleEntity (SPARK_PARTICLE, ParticleEntities[SPARK_PARTICLE]);
  PR_SetParticleDrawProc (SPARK_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (SPARK_EMITTER);
  PR_SetEmitterTimeLimit (SPARK_EMITTER, 1);
  PR_SetEmitterNumParticles (SPARK_EMITTER, 15);
  PR_SetEmitterParticleType (SPARK_EMITTER, SPARK_PARTICLE);
  PR_SetEmitterRandomVelocity (SPARK_EMITTER, 10);


  /* 3D Mesh Explosion */
  PR_InitializeParticleType (EXPLOSION1_PARTICLE);
  PR_SetParticleAlpha (EXPLOSION1_PARTICLE, 255, -4);
  PR_SetParticleScale (EXPLOSION1_PARTICLE, 0.3f, 0.023f);
  PR_SetParticleEntity (EXPLOSION1_PARTICLE, ParticleEntities[EXPLOSION1_PARTICLE]);
  PR_SetParticleDrawProc (EXPLOSION1_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (EXPLOSION1_EMITTER);
  PR_SetEmitterTimeLimit (EXPLOSION1_EMITTER, 1);
  PR_SetEmitterParticleType (EXPLOSION1_EMITTER, EXPLOSION1_PARTICLE);


  bouncefactor.x = 1.0f;
  bouncefactor.y = -0.55f;
  bouncefactor.z = 1.0f;
  PR_InitializeParticleType (BOUNCEPUFF_PARTICLE);
  PR_SetParticleAlpha (BOUNCEPUFF_PARTICLE, 255, -5);
  PR_SetParticleScale (BOUNCEPUFF_PARTICLE, 0.3f, 0.03f);
  PR_SetParticleEntity (BOUNCEPUFF_PARTICLE, ParticleEntities[BOUNCEPUFF_PARTICLE]);
  PR_SetParticleDrawProc (BOUNCEPUFF_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (BOUNCEPUFF_EMITTER);
//  PR_SetEmitterTimeLimit (BOUNCEPUFF_EMITTER, 500);
  PR_SetEmitterParticleType (BOUNCEPUFF_EMITTER, BOUNCEPUFF_PARTICLE);
  PR_SetEmitterBounceParameters (BOUNCEPUFF_EMITTER, &bouncefactor, 5);
  PR_SetEmitterGravity (BOUNCEPUFF_EMITTER, -0.43);
  PR_SetEmitterMovement (BOUNCEPUFF_EMITTER, 1);
  PR_SetEmitterCollisionProc (BOUNCEPUFF_EMITTER, BouncePuffCollide);
  PR_SetEmitterDelay (BOUNCEPUFF_EMITTER, 2);



  /* Ring Mesh Explosion */
  PR_InitializeParticleType (EXPLOSION2_PARTICLE);
  PR_SetParticleAlpha (EXPLOSION2_PARTICLE, 255, -4);
  PR_SetParticleScale (EXPLOSION2_PARTICLE, 0.05f, 0.073f);
  PR_SetParticleEntity (EXPLOSION2_PARTICLE, ParticleEntities[EXPLOSION2_PARTICLE]);
  PR_SetParticleDrawProc (EXPLOSION2_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (EXPLOSION2_EMITTER);
  PR_SetEmitterTimeLimit (EXPLOSION2_EMITTER, 1);
  PR_SetEmitterParticleType (EXPLOSION2_EMITTER, EXPLOSION2_PARTICLE);
}


void InitializeWeapons (void)
{
  CreateParticleMaterials ();
  InitializeParticleTemplates ();
}


void FireEmitter (PR_DWORD emitnum, PR_ENTITY *src,
                  PR_POINT *offset, PR_POINT *velocity)
{
PR_ORIENTATION *orient;
PR_POINT location, dvector;

  /* Get a pointer to the entity orientation */
  orient = &src->orientation;


  /* Find the initial location by rotating the offset vector by the
     source's rotation, and adding the center of the source. */
  tvector[0] = offset->x;
  tvector[1] = offset->y;
  tvector[2] = offset->z;
  PR_Transform (orient->rot_mat); 
  location.x = tvector[0] + orient->location.x;
  location.y = tvector[1] + orient->location.y;
  location.z = tvector[2] + orient->location.z;


  /* Find the velocity by rotating the velocity vector by the source's
     rotation.  The original velocity defines the direction of the emitter
     if the source is aiming straight ahead.  It also defines the speed of
     the emitter per frame */
  tvector[0] = velocity->x;
  tvector[1] = velocity->y;
  tvector[2] = velocity->z;
  PR_Transform (orient->rot_mat); 
  dvector.x = tvector[0];
  dvector.y = tvector[1];
  dvector.z = tvector[2];

  PR_CreateEmitter (emitnum, &location, &dvector);
}


void FireWeapon1 (PR_ENTITY *entity)
{
  if (shoot_wing)
    FireEmitter (FLARETRAIL_EMITTER, entity, &WeaponOffsets[0],
                 &WeaponVectors[0]);
  else
    FireEmitter (FLARETRAIL_EMITTER, entity, &WeaponOffsets[1],
                 &WeaponVectors[1]);
  shoot_wing = !shoot_wing;  /* alternate between 0 and 1 */

  PRSND_Set3DSoundCoordinate (sound_weap1,
                              spaceship_entity->orientation.location.x,
                              spaceship_entity->orientation.location.y,
                              spaceship_entity->orientation.location.z,
                              spaceship_entity);
  PRSND_Play3DSound (sound_weap1, Waves[WAVE_WEAPON1]);
  PRSND_Update3DSound (sound_weap1);
  PRSND_UpdateVoices ();
}

