#include <math.h>
#include <malloc.h>
#include <stdlib.h>
#include <direct.h>
#include <dos.h>
#include <conio.h>
#include <string.h>
#include <io.h>

#include <pr.h>
#include "lslib.h"

#if defined (MSGLIDE) || defined (WTGLIDE)
 #include <glide.h>
 #include <pr3dfx.h>
 #include "winutil.h"
 #include <windows.h>
#elif defined (WIN32)
 #include "winutil.h"
 #include <windows.h>
#else
 #ifdef __3DFX__
  #include <glide.h>
  #include <pr3dfx.h>
  #include <fxgui.h>
 #endif
#endif


void help_log_message (char *msg, ... );


PR_TERRAIN * terrain;
terrain_settings_struct Terrain_Settings;

PR_DWORD     NumBaseTiles;
PR_DWORD     tilemap[256*2];
block        tilesprites[1024];
TileProperty TileProps[512];

ObjectType * ObjectList;
PR_DWORD     TopEntity;
EntityType * EntityList;

PR_DWORD MaxObjects = 50;
PR_DWORD MaxEntities = 1000;
PR_DWORD ObjectsLoaded = 0;

static void LS_LoadLevelInfo (void)
{
FILE *in;
char title[28] = "Landscape Studio Project\n";
char project_name[10];
PR_DWORD version;
PR_UWORD chunk;
PR_DWORD size;
PR_DWORD tilenum, temp, objnum;

  /* Library code */
  if (wgtlibrary == NULL)
    {
     libf = fopen ("terrain.lvl", "rb");
     if (libf == NULL)
       return;
     in = libf;
    }
  else
    {
     if ((libf = fopen (wgtlibrary, "rb")) == NULL)
       return;
     readheader ();
     findfile ("terrain.lvl");
     if (lresult == 1)
       fseek (libf, lfpos, SEEK_SET);
     else
       {
        fclose (libf);
        return;
       }
     in = libf;
    }

  fread (&title, 25, 1, in);
  fread (&version, 4, 1, in);
  fread (project_name, 8, 1, in);

  tilenum = 0;
  objnum = 0;

  do {
    fread (&chunk, 2, 1, in);
    fread (&size, 4, 1, in);

    switch (chunk)
      {
       case 0x1001:
         fread (&Terrain_Settings.TileScale, 4, 1, in);
         break;

       case 0x1002:
         fread (&Terrain_Settings.HeightScale, 4, 1, in);
         break;

       case 0x1003:
         fread (&Terrain_Settings.Radius, 4, 1, in);
         break;

       case 0x1004:
         fread (&Terrain_Settings.Layers, 4, 1, in);
         break;

       case 0x1005:
         fread (&Terrain_Settings.FogEnabled, 4, 1, in);
         fread (&Terrain_Settings.FogNear, 4, 1, in);
         fread (&Terrain_Settings.FogFar, 4, 1, in);
         fread (&Terrain_Settings.FogR, 1, 1, in);
         fread (&Terrain_Settings.FogG, 1, 1, in);
         fread (&Terrain_Settings.FogB, 1, 1, in);
         break;

       case 0x1006:
         fread (&Terrain_Settings.Mirror, 4, 1, in);
         break;

/*
       case 0x1501:
         fread (&Camera_Settings.vec, 12, 1, in);
         fread (&Camera_Settings.zoom, 4, 1, in);
         fread (&Camera_Settings.height, 4, 1, in);
         fread (&Camera_Settings.destheight, 4, 1, in);
         break;
*/

/*
       case 0x1502:
         fread (&view_x, 4, 1, in);
         fread (&view_y, 4, 1, in);
         fread (&edit_x, 4, 1, in);
         fread (&edit_y, 4, 1, in);
         fread (&edit_x2, 4, 1, in);
         fread (&edit_y2, 4, 1, in);
         fread (&edit_tile, 4, 1, in);
         fread (&edit_height, 4, 1, in);
         fread (&edit_shade, 4, 1, in);

         fread (&layer_active[0], 4, 3, in);
         fread (&tile_active[0], 4, 3, in);
         fread (&edit_mode, 4, 1, in);
         break;
*/

       /* Tile info */
       case 0x2000:
         fread (&NumBaseTiles, 4, 1, in);
         break;

       case 0x2001:
         fread (&temp, 4, 1, in);
         break;

       case 0x2010:
         fread (&TileProps[tilenum].keyed, 4, 1, in);
         break;

       case 0x2011:
         fread (&TileProps[tilenum].translucent, 4, 1, in);
         break;

       case 0x2012:
         fread (&TileProps[tilenum].r, 1, 1, in);
         fread (&TileProps[tilenum].g, 1, 1, in);
         fread (&TileProps[tilenum].b, 1, 1, in);
         fread (&TileProps[tilenum].a, 1, 1, in);
         break;

       case 0x2200:
         tilenum++;
         break;

       /* Object info */
       case 0x3000:
         fread (&TopEntity, 4, 1, in);
         break;

       case 0x3010:
         fread (&EntityList[objnum].rot_mat, 64, 1, in);
         break;

       case 0x3011:
         fread (&EntityList[objnum].location, 12, 1, in);
         break;

       case 0x3012:
         fread (&EntityList[objnum].scale, 12, 1, in);
         break;

       case 0x3013:
         fread (&EntityList[objnum].layer, 4, 1, in);
         break;

       case 0x3014:
         fread (&EntityList[objnum].flags, 4, 1, in);
         break;

       case 0x3015:  /* Key field */
         fread (&EntityList[objnum].name, 13, 1, in); 
         break;

       case 0x3200:
         objnum++;
         break;


       default:
        fseek (in, ftell (in) + size, 0);
      }
  } while (chunk != 0xEEEE);

  fclose (in);
}


static void InitializeTerrainSettings (void)
{
  Terrain_Settings.FogColor = PRGFX_MakeColor (Terrain_Settings.FogR,
                                               Terrain_Settings.FogG,
                                               Terrain_Settings.FogB);

  PR_SetFogColor (Terrain_Settings.FogColor);
  PR_SetFogRange (Terrain_Settings.FogNear, Terrain_Settings.FogFar);

  if (Terrain_Settings.FogEnabled)
    PR_SetFogState (TRUE);
  else
    PR_SetFogState (FALSE);
}





static void UpdateMaterial (PR_DWORD matnum)
{
PR_MATERIAL *m;

  m = &PR_ObjectMaterialList[matnum];

  if (TileProps[matnum].translucent)
    {
     if (TileProps[matnum].keyed)
       PR_SetMaterialMethod (m, T_FOG_XTRANSLUCENT_TEXTURED);
     else
       PR_SetMaterialMethod (m, T_FOG_WATER_SPECIAL);
    }
  else
    {
     if (TileProps[matnum].keyed)
       PR_SetMaterialMethod (m, T_FOG_XGOURAUD_TEXTURED);
     else
       PR_SetMaterialMethod (m, T_FOG_GOURAUD_TEXTURED);
    }

  PR_SetMaterialColor (m, TileProps[matnum].r,
                          TileProps[matnum].g,
                          TileProps[matnum].b);
  PR_SetMaterialAlpha (m, TileProps[matnum].a);
}




static void InitializeTerrainMaterials (void)
{
PR_MATERIAL *m;
PR_DWORD i;
char tilename[80];

  for (i = 1; i < 1024; i++)
	tilesprites[i] = NULL;

  wloadsprites (global_palette, "tiles.spr", tilesprites, 0, 511);
  /* Load the images into the tilesprites array */

  if (tilesprites[0] != NULL)
    NumBaseTiles = wgetblockwidth (tilesprites[0]);

  /* Set the null type for land tiles
     This is to allow for holes in the landscape. */
  m = &PR_ObjectMaterialList[0];
  PR_SetMaterialName (m, "NULL");
  PR_SetMaterialMethod (m, NULL_TYPE);


  for (i = 1; i < 512; i++)
    {
     m = &PR_ObjectMaterialList[i];

     if (tilesprites[i] != NULL)
       {
        /* We already have the images loaded in tilesprites, so just
           make a pointer to them */
        sprintf (tilename, "tile%i", i-1);
        if (tilesprites[i] != NULL)
          tilemap[i] = PR_AddTexture (tilename, tilesprites[i]);

        /* This is the master material.  Mip mapping information is entered
          in here only.  */
        m = &PR_ObjectMaterialList[i];
        PR_SetMaterialName (m, "tile1");

        PR_SetMaterialMethod (m, 72);

        PR_SetMaterialBaseColor (m, 0);
        PR_SetMaterialColor (m, 255, 255, 255);
        PR_SetMaterialTexture (m, tilemap[i]);
        PR_SetMaterialTable (m, 0);
        PR_SetMaterialShades (m, 31);
        PR_SetMaterialEnvironmentMap (m, 0);
        PR_SetMaterialMipMapState (m, FALSE);
        PR_SetMaterialAlpha (m, 255);

        UpdateMaterial (i);
       }
    }
}







static void LS_CreateTerrain (void)
{
PR_DWORD flags;

  flags = TERRAIN_SHADING;
  if (Terrain_Settings.Layers > 2)
    flags |= TERRAIN_3LEVEL;
  else if (Terrain_Settings.Layers > 1)
    flags |= TERRAIN_2LEVEL;
  else 
    flags |= TERRAIN_PLAIN;

  terrain = PR_AllocateTerrain (Terrain_Settings.Radius,
                                flags,
                                Terrain_Settings.TileScale,
                                Terrain_Settings.HeightScale,
                                0);

  PR_LoadTerrain (terrain, 1, "height1.pcx", "map1.wmp", "shade1.pcx", 32);
  if (Terrain_Settings.Layers > 1)
    PR_LoadTerrain (terrain, 2, "height2.pcx", "map2.wmp", "shade2.pcx", 32);
  if (Terrain_Settings.Layers > 2)
    PR_LoadTerrain (terrain, 3, "height3.pcx", "map3.wmp", "shade3.pcx", 32);
}



PR_DWORD LS_FindObject (char *filename)
/* -----------------------------------------------------------------------
   Function:   LS_FindObject
   Purpose:    Finds the object number given its filename
   Parameters: filename - Filename of the object
   Return:     OBJECT_NOT_FOUND if the file is not in the ObjectList array
               or object number
   ----------------------------------------------------------------------- */
{
PR_DWORD i;
char *name;

  if (filename == NULL)
    return (OBJECT_NOT_FOUND);
  strupr (filename);
  
  if (MaxObjects == 0)
    return (OBJECT_NOT_FOUND);

  for (i = 0; i < MaxObjects; i++)
    if (ObjectList[i].name != NULL)
      {
       name = strdup (ObjectList[i].name);
       strupr (name);
    
       if (strcmp (name, filename) == 0)
         {
          free (name);
          return (i);
         }
       free (name);
     }
  return (OBJECT_NOT_FOUND);
}




PR_DWORD LS_LoadObject (char *filename)
/* -----------------------------------------------------------------------
   Function:   LS_LoadObject
   Purpose:    Loads the object and returns the object number
   Parameters: filename - Filename of the object
   Return:     Index of the object
   ----------------------------------------------------------------------- */
{
PR_DWORD i;
PR_DWORD found;

  found = -1;

  /* Find an empty position */
  for (i = 0; i < MaxObjects; i++)
    if (ObjectList[i].name == NULL)
      {
       found = i;
       break;
      }

  if (found == -1)
    return -1;


  ObjectList[found].obj = PR_LoadPRO (filename, LOAD_NORMAL);
  if (ObjectList[found].obj != NULL)
    {
     ObjectList[found].name = strdup (filename);
     ObjectList[found].lastscale.x = 1.0f;
     ObjectList[found].lastscale.y = 1.0f;
     ObjectList[found].lastscale.z = 1.0f;
     PR_MatrixIdentity (ObjectList[found].lastrot);

     return found;
    }

  return -1;
}

/*
void SearchForObjects (void)
{
struct find_t fileinfo;
unsigned rc;

  rc = _dos_findfirst("*.PRO", _A_NORMAL, &fileinfo);
  while (rc == 0)
    {
     // load fileinfo.name
     ObjectsLoaded = LS_LoadObject (fileinfo.name);
     rc = _dos_findnext (&fileinfo);
    }

  ObjectsLoaded++;
}

*/

void SearchForObjects (void)
{
#ifdef __WATCOMC__
struct find_t fileinfo;
#else
struct _finddata_t fileinfo;
long fhandle;
#endif

unsigned rc = 0;

  #ifdef __WATCOMC__
    rc = _dos_findfirst("*.PRO", _A_NORMAL, &fileinfo);
  #else
    fhandle = _findfirst("*.PRO", &fileinfo);
    if (fhandle == -1)
      rc = -1;
  #endif

  while (rc == 0)
    {
     // load fileinfo.name
     ObjectsLoaded = LS_LoadObject (fileinfo.name);

     #ifdef __WATCOMC__
       rc = _dos_findnext (&fileinfo);
     #else
       rc = _findnext (fhandle, &fileinfo);
     #endif
    }

  _findclose (fhandle);
  ObjectsLoaded++;
}


void InitializeAllEntities (void)
{
PR_DWORD entnum, object_number;

  for (entnum = 0; entnum < TopEntity; entnum++)
    {
     /* Create the entity and initialize it with the current settings */
     object_number = LS_FindObject (EntityList[entnum].name);
     if (object_number != OBJECT_NOT_FOUND)
       {
        EntityList[entnum].ent = PR_CreateEntity (ObjectList[object_number].obj, "Ent");
        EntityList[entnum].objnum = object_number;

        memcpy (EntityList[entnum].ent->orientation.rot_mat,
                EntityList[entnum].rot_mat, sizeof (PR_MATRIX));
        PR_PositionEntity (EntityList[entnum].ent,
                           EntityList[entnum].location.x,
                           EntityList[entnum].location.y,
                           EntityList[entnum].location.z);
        PR_ScaleEntityAbs (EntityList[entnum].ent,
                           EntityList[entnum].scale.x,
                           EntityList[entnum].scale.y,
                           EntityList[entnum].scale.z);
       }
     else
      EntityList[entnum].flags = 0;  /* Not loaded */
    }
}




void LS_AllocObjects (void)
/* -----------------------------------------------------------------------
   Function:   LS_AllocObjects
   Purpose:    Allocates room for a list of objects
   Parameters: maxobj - Number of objects to allocate space for
   Return:     None
   ----------------------------------------------------------------------- */
{
PR_DWORD i;

  ObjectList = (ObjectType *)malloc (sizeof (ObjectType) * MaxObjects);

  for (i = 0; i < MaxObjects; i++)
    {
     ObjectList[i].obj = NULL;
     ObjectList[i].name = NULL;
    }
}



void LS_AllocEntities (void)
/* -----------------------------------------------------------------------
   Function:   AllocEntities
   Purpose:    Allocates room for a list of objects
   Parameters: maxobj - Number of objects to allocate space for
   Return:     None
   ----------------------------------------------------------------------- */
{
PR_DWORD i;

  EntityList = (EntityType *)malloc (sizeof (EntityType) * MaxEntities);

  for (i = 0; i < MaxEntities; i++)
    {
     PR_MatrixIdentity (EntityList[i].rot_mat);
     EntityList[i].location.x = 0.0f;
     EntityList[i].location.y = 0.0f;
     EntityList[i].location.z = 0.0f;

     EntityList[i].scale.x = 1.0f;
     EntityList[i].scale.y = 1.0f;
     EntityList[i].scale.z = 1.0f;

     EntityList[i].layer = 0;
     EntityList[i].flags = 0;
     EntityList[i].name[0] = 0;
    }
}




void LS_LoadTerrain (char *filename)
/* -----------------------------------------------------------------------
   Function:   LS_LoadTerrain
   Purpose:    Loads a Landscape Studio level
   Parameters: filename - Name of the level file (.lsl)
   Return:     None
   ----------------------------------------------------------------------- */
{
help_log_message ("lt 1");
  setlib (filename);
help_log_message ("lt 2");

  LS_AllocObjects ();
help_log_message ("lt 3");
  LS_AllocEntities ();
help_log_message ("lt 4");

  LS_LoadLevelInfo ();
help_log_message ("lt 5");
  LS_CreateTerrain ();
help_log_message ("lt 6");
  InitializeTerrainMaterials ();
help_log_message ("lt 7");
  InitializeTerrainSettings ();
help_log_message ("lt 8");

  setlib (NULL);
help_log_message ("lt 9");
  PR_SetTextureFormat (TEXTURE_P_8);
help_log_message ("lt 10");
  SearchForObjects ();
help_log_message ("lt 11");
  InitializeAllEntities ();
help_log_message ("lt 12");
}





void LS_EntityCoordinateFix (PR_ENTITY *ent)
/* -----------------------------------------------------------------------
   Function:   LS_EntityCoordinateFix
   Purpose:    Keeps the distance of from entity to the camera within half
               the map dimensions
   Parameters: ent - Pointer to the entity
   Return:     None
   ----------------------------------------------------------------------- */
{
PR_DWORD dx, dz;
PR_DWORD hx, hz;

  dx = ent->orientation.location.x - PR_CurrentCamera->source.x;
  dz = ent->orientation.location.z - PR_CurrentCamera->source.z;

  hx = (terrain->width >> 1) * terrain->world_scale;
  hz = (terrain->height >> 1) * terrain->world_scale;


  if (dx > hx)
    ent->orientation.location.x -= terrain->width * terrain->world_scale;
  if (dx < -hx)
    ent->orientation.location.x += terrain->width * terrain->world_scale;

  if (dz > hz)
    ent->orientation.location.z -= terrain->height * terrain->world_scale;
  if (dz < -hz)
    ent->orientation.location.z += terrain->height * terrain->world_scale;
}



void LS_RenderObjects (void)
/* -----------------------------------------------------------------------
   Function:   LS_RenderObjects
   Purpose:    Displays the objects in the level
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */
{
PR_DWORD i;
PR_DWORD dx, dz;
PR_REAL radius;
PR_ENTITY *ent;

  radius = Terrain_Settings.TileScale * Terrain_Settings.Radius;


  for (i = 0; i < TopEntity; i++)
    {
     ent = EntityList[i].ent;
     if (EntityList[i].flags & ENTITY_LOADED)
       {
        LS_EntityCoordinateFix (ent);
        dx = ent->orientation.location.x - PR_CurrentCamera->source.x;
        dz = ent->orientation.location.z - PR_CurrentCamera->source.z;

        if ((dx > -radius) && (dx < radius) &&
            (dz > -radius) && (dz < radius))
          {
           // PR_AnimateEntity (ent, ent->userdata[0]);

           PR_TransformEntity (EntityList[i].ent);
           PR_RenderEntity (EntityList[i].ent);

           if (Terrain_Settings.Mirror)
             {
              PR_Settings.FlipNormals = 1;
              PR_PostMatrix[5] = -1.0f;
              PR_PostMatrix[13] = 254.0f * Terrain_Settings.HeightScale;    /* Y translation */
              PR_TransformEntity (EntityList[i].ent);
              PR_RenderEntity (EntityList[i].ent);
              PR_PostMatrix[5] = 1.0f;
              PR_PostMatrix[13] = 0.0f;
              PR_Settings.FlipNormals = 0;
             }
          }
       }
    }
}

