#include <conio.h>
#include <stdlib.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 3

   Description:  Same as example 2, but with lights and shaded texture
                 mapping.
*/

                                            
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *entity;                     /* A single entity of the shape */
PR_LIGHTLIST userlights;               /* Holds a list of lights */

PR_DWORD vwidth, vheight;
PR_REAL cenx, ceny, cenz;
PR_UDWORD BLACK;                       /* Color index/value for black */


void AppOpen (void)
{
  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */

  PR_Settings.UseWindowed = TRUE;

  /* This should match the size of the original
     window in rushutil.c */
  if (PR_Settings.UseWindowed)  
    {
	 vwidth = 320;
	 vheight = 200;
	}
  else
    {
     vwidth = PR_VideoModes[0].width;
     vheight = PR_VideoModes[0].height;
	}
  PR_SetMode (vwidth, vheight, 60);

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  /* Load the object */
  PR_SetTexturePath ("..\\data\\");
  PR_SetTablePath ("..\\data\\");
  object_shape = PR_LoadPRO ("..\\data\\cube2.pro", LOAD_NORMAL);
  PR_CenterObject (object_shape, &cenx, &ceny, &cenz);

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  entity = PR_CreateEntity (object_shape, "TEST ENTITY");


  /* Initialize the camera */
  camera = PR_AllocCamera ();
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, 400);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

}


void AppUpdate (void)
{
  PR_OpenScreen (PR_BACKBUFFER);

  PRGFX_SetColor (BLACK);
  PRGFX_ClearScreen ();             /* Clear the last frame */

  PR_NewFrame ();                   /* Begin a new frame */

  PR_TransformLights (&userlights);

  PR_RotateEntity (entity, 1.5, 1.2, 0.7);
  PR_TransformEntity (entity);      /* Transform the entity */
  PR_RenderEntity (entity);         /* Render the entity */

  PR_RenderFrame ();                /* Draw the scene */

  PR_Flip (1);                      /* Show the background page */
}



void AppClose (void)
{
  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (entity);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();

  PR_ShutDown ();
}

