#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <pr.h>
#include <prgui.h>
#include "mdl.h"

/* Most of this code was written by Trey Harrison of Crack Dot Com.
   (trey@crack.com)

   I have modified it in a few ways:
     - Premultiplies the scale and offset into each vertex array instead
       of each time it draws a frame
     - Pre-evaluated the gl strips and fans into a triangle list so
       Power Render can render it.
     - Removed all his texture and triangle drawing stuff because PR
       handles that.

   A big thanks goes Trey for releasing this code and inspiring me to
   make a better Quake model viewer.

*/


PR_VERTEX *frame_list[256];
mdl_t     *testmdl;
mdl_t2    *testmd2;
extern PR_OBJECT *test;                /* The shape loaded */
extern char pcxfile[80];               /* Name of PCX file if MD2 requires it */
extern char pakfile[80];

#define GOURAUD_MODE 1
extern PR_DWORD shade_mode;

mdlheader_t2 md2header;



void FixMDLNormals (int numframes)
{
PR_DWORD i, frame;
PR_VERTEX *frameptr;
PR_REAL cenx, ceny, cenz;
PR_SEGMENT *seg;

  PR_CenterObject (test, &cenx, &ceny, &cenz);
  seg = test->segment_list;

  for (frame = 0; frame < numframes; frame++)
    {
     for (i = 0; i < test->segment_list->num_vertices; i++)
       {
        frameptr = frame_list[frame];
        frameptr[i].x -= cenx;
        frameptr[i].y -= ceny;
        frameptr[i].z -= cenz;
       }

     memcpy (seg->vertex_list, frame_list[frame], seg->num_vertices * sizeof (PR_VERTEX));
     PR_InitializeFaceNormals (test);
     PR_InitializeVertexNormals (test);
     memcpy (frame_list[frame], seg->vertex_list, seg->num_vertices * sizeof (PR_VERTEX));
    }
}



char *remove_path(char *src)
{
  char *ret = src;

  while (*src)
  {
    if ((*src=='/') || (*src=='\\'))
      ret=src+1;

    src++;    
  }

  return ret;
}



FILE *FindInPath (char *filename, int *size, int *offset)
{  
FILE *f;

  filename = remove_path (filename);

  f = fopen (filename, "rb");
  
  if (!f)
    return NULL;
  
  fseek (f, 0, SEEK_END);
  *size   = ftell (f);
  fseek (f, 0, SEEK_SET);
  
  *offset = 0;
  
  return f;
}



FILE *FindInPak (char *pakname, char *filename, int *size, int *offset)
{
int          num_pak_entries;
pakentry_t  *pak_entries;
pakheader_t  pakheader;
int          i,found;
FILE        *f; 

  if ((pakname == NULL) || (pakname[0] == 0))
    PRGUI_Error ("Invalid pakfile (use the -m switch)");
  
  f = fopen (pakname, "rb");
  
  if (!f)
    return FindInPath (filename,size,offset);

  fread (&pakheader, sizeof(pakheader_t), 1, f);
  
  fseek (f, pakheader.diroffset, SEEK_SET);
  num_pak_entries = pakheader.dirsize / (sizeof(pakentry_t));

  pak_entries = (pakentry_t *)malloc(sizeof(pakentry_t) * num_pak_entries);
  fread (pak_entries, pakheader. dirsize, 1, f);

  found = 0;

  for (i = 0; i < num_pak_entries; i++)
    {
     if (!stricmp (pak_entries[i].filename, filename))
       {
        found = 1;
        break;
       }
    }  

  if (!found)
    {
     free(pak_entries);
     return FindInPath (filename,size,offset);    
    }  

  fseek (f, pak_entries[i].offset, SEEK_SET);

  *size   = pak_entries[i].size;
  *offset = pak_entries[i].offset;

  free (pak_entries);  
  return f;
}





void PR_ImportMDLTextures (mdl_t *mdl)
{
PR_MATERIAL *m;
PR_DWORD matnum;
skin_t *skin;
block skinpic;
char texname[20];
unsigned char *bits;
PR_DWORD pitch, y;

  wloadpalette ("quake.pal", global_palette);

  PR_LoadTable ("quake.tab");
  PR_SetTextureFormat (TEXTURE_NORMAL);

  for (matnum = 0; matnum < mdl->numskins; matnum++)
    {
     skin = &mdl->skins[matnum];

     /* Create the texture */
     skinpic = wallocblock (mdl->skinwidth, mdl->skinheight);
 
     #if defined (MSDD) || defined (WTDD)
     bits = wgetblocksurface (skinpic, &pitch);
     if (bits == NULL)
        return;
     #else
     bits = skinpic + 4;
     pitch = mdl->skinwidth;
     #endif

     for (y = 0; y < mdl->skinheight; y++)
       {
        memcpy (bits, skin->skin[0] + y * mdl->skinwidth, mdl->skinwidth);
        bits += pitch;
       }
     
     #if defined (MSDD) || defined (WTDD)
        wunlocksurface (skinpic);
     #endif
     
     sprintf (texname, "skin%i.pcx", matnum);
  //   wsavepcx (texname, skinpic, global_palette);

     
     m = &PR_ObjectMaterialList[matnum];

     PR_SetMaterialName (m, texname);
     if (shade_mode == GOURAUD_MODE)
       PR_SetMaterialMethod (m, T_GOURAUD_TEXTURED8);
     else
       PR_SetMaterialMethod (m, T_TEXTURED);
     PR_SetMaterialBaseColor (m, 0);
     PR_SetMaterialColor (m, 255, 255, 255);
     PR_SetMaterialTexture (m, PR_AddTexture (texname, skinpic));
     PR_SetMaterialTable (m, 0);
     PR_SetMaterialShades (m, 31);
     PR_SetMaterialEnvironmentMap (m, 0);
     PR_SetMaterialMipMapState (m, FALSE);
     m->a = 255;

     PR_AddMaterial (m);
    }
}



PR_OBJECT *PR_ImportMDL (mdl_t *mdl)
/* Converts an MDL structure to a PR_OBJECT structure */
{
PR_OBJECT *obj;
PR_SEGMENT *seg;
PR_FACE_DATA_REAL *datareal;
PR_FACE_DATA *datafix;
PR_DWORD texnum;
PR_DWORD width, height;
PR_DWORD i, frame;
frame_t *frameptr;
PR_VERTEX *vptr;

PR_DWORD facesfront, v1OnSeam, v2OnSeam, v3OnSeam;
PR_REAL sadd;
PR_REAL xs,ys;

  PR_ImportMDLTextures (mdl);


  /* Allocate the object */
  obj = PR_AllocObject ();

  /* Allocate the segment */
  obj->segment_list = PR_AllocSegment ();
  obj->num_segments = 1;
  obj->num_frames = -1;

  /* Allocate the faces */
  seg = obj->segment_list;
  seg->num_faces = mdl->numtris;
  seg->face_list = malloc (seg->num_faces * sizeof (PR_FACE));
  memset (seg->face_list, 0, seg->num_faces * sizeof (PR_FACE));

  seg->num_vertices = mdl->numverts;
  seg->vertex_list = malloc (seg->num_vertices * sizeof (PR_VERTEX));
  seg->vertex_data = malloc (seg->num_vertices * sizeof (PR_VERTEX_DATA));
  memset (seg->vertex_list, 0, seg->num_vertices * sizeof (PR_VERTEX));
  memset (seg->vertex_data, 0, seg->num_vertices * sizeof (PR_VERTEX_DATA));

  seg->flags = 0;

  /* Set up vertices, normals, and face data */

  for (frame = 0; frame < mdl->numframes; frame++)
    {
     frame_list[frame] = malloc (seg->num_vertices * sizeof (PR_VERTEX));
     memset (frame_list[frame], 0, seg->num_vertices * sizeof (PR_VERTEX));

     frameptr = &mdl->frames[frame];
     vptr = frame_list[frame];

     for (i = 0; i < seg->num_vertices; i++)
       {
        vptr[i].z = frameptr->data[0][i].x * mdl->scale.x * 50.0f;
        vptr[i].x = frameptr->data[0][i].y * mdl->scale.y * 50.0f;
        vptr[i].y = frameptr->data[0][i].z * mdl->scale.z * 50.0f;
        vptr[i].vdata = &seg->vertex_data[i];
       }
    }


   memcpy (seg->vertex_list, frame_list[0], seg->num_vertices * sizeof (PR_VERTEX));

   frame = 0;
   frameptr = &mdl->frames[0];
   for (i = 0; i < seg->num_faces; i++)
     {
     seg->face_list[i].vertex1 = PR_GetVertexPointer (seg->vertex_list, mdl->triangles[i].vertices[0]);
     seg->face_list[i].vertex2 = PR_GetVertexPointer (seg->vertex_list, mdl->triangles[i].vertices[1]);
     seg->face_list[i].vertex3 = PR_GetVertexPointer (seg->vertex_list, mdl->triangles[i].vertices[2]);

     seg->face_list[i].material = 0;
     seg->face_list[i].backmaterial = 0;

     seg->face_list[i].flags = FFLAG_FRONT_VISIBLE; 


     /* Set up the texture coordinates */
     texnum = PR_ObjectMaterialList[0].texture_number;
     width = PR_WorldTextures[texnum].width;
     height = PR_WorldTextures[texnum].height;

     xs = ((PR_REAL)width  / (PR_REAL)mdl->skinwidth);
     ys = ((PR_REAL)height / (PR_REAL)mdl->skinheight);


     if (PR_Settings.RealUV)
       {
        datareal = (PR_FACE_DATA_REAL *)&seg->face_list[i].face_data;

        facesfront = mdl->triangles[i].facesfront;
        v1OnSeam = mdl->verts[mdl->triangles[i].vertices[0]].onseam;
        v2OnSeam = mdl->verts[mdl->triangles[i].vertices[1]].onseam;
        v3OnSeam = mdl->verts[mdl->triangles[i].vertices[2]].onseam;

        if (PR_OutputDevice.devicetype == DEVICE_D3D)
          {
           xs /= width;
           ys /= height;
          }
        if (PR_OutputDevice.devicetype == DEVICE_D3D)
          sadd = (v1OnSeam && !facesfront)?(0.5f):(0.0f);
        else 
          sadd = (v1OnSeam && !facesfront)?((PR_REAL)PR_WorldTextures[texnum].width/2.0f):(0.0f);

        datareal->u[0] = ((PR_REAL)mdl->verts[mdl->triangles[i].vertices[0]].s * xs + sadd);
        datareal->v[0] = ((PR_REAL)mdl->verts[mdl->triangles[i].vertices[0]].t * ys);

        if (PR_OutputDevice.devicetype == DEVICE_D3D)
          sadd = (v2OnSeam && !facesfront)?(0.5f):(0.0f);
        else 
          sadd = (v2OnSeam && !facesfront)?((PR_REAL)PR_WorldTextures[texnum].width/2.0f):(0.0f);
        datareal->u[1] = ((PR_REAL)mdl->verts[mdl->triangles[i].vertices[1]].s * xs + sadd);
        datareal->v[1] = ((PR_REAL)mdl->verts[mdl->triangles[i].vertices[1]].t * ys);

        if (PR_OutputDevice.devicetype == DEVICE_D3D)
          sadd = (v3OnSeam && !facesfront)?(0.5f):(0.0f);
        else 
          sadd = (v3OnSeam && !facesfront)?((PR_REAL)PR_WorldTextures[texnum].width/2.0f):(0.0f);
        datareal->u[2] = ((PR_REAL)mdl->verts[mdl->triangles[i].vertices[2]].s * xs + sadd);
        datareal->v[2] = ((PR_REAL)mdl->verts[mdl->triangles[i].vertices[2]].t * ys);
       }
      else
       {
        datafix = &seg->face_list[i].face_data;

        facesfront = mdl->triangles[i].facesfront;
        v1OnSeam = mdl->verts[mdl->triangles[i].vertices[0]].onseam;
        v2OnSeam = mdl->verts[mdl->triangles[i].vertices[1]].onseam;
        v3OnSeam = mdl->verts[mdl->triangles[i].vertices[2]].onseam;

        sadd = (v1OnSeam && !facesfront)?((PR_REAL)PR_WorldTextures[texnum].width/2.f):(0);
        datafix->u[0] = (PR_DWORD)(mdl->verts[mdl->triangles[i].vertices[0]].s + sadd) << 16;
        datafix->v[0] = (PR_DWORD)(mdl->verts[mdl->triangles[i].vertices[0]].t) << 16;

        sadd = (v2OnSeam && !facesfront)?((PR_REAL)PR_WorldTextures[texnum].width/2.f):(0);
        datafix->u[1] = (PR_DWORD)(mdl->verts[mdl->triangles[i].vertices[1]].s + sadd) << 16;
        datafix->v[1] = (PR_DWORD)(mdl->verts[mdl->triangles[i].vertices[1]].t) << 16;

        sadd = (v3OnSeam && !facesfront)?((PR_REAL)PR_WorldTextures[texnum].width/2.f):(0);
        datafix->u[2] = (PR_DWORD)(mdl->verts[mdl->triangles[i].vertices[2]].s + sadd) << 16;
        datafix->v[2] = (PR_DWORD)(mdl->verts[mdl->triangles[i].vertices[2]].t) << 16;
       }
    }

  PR_ObjectCalcBoundingBox (obj);
  PR_FlipFaceNormals (obj);
  PR_InitializeFaceNormals (obj);
  PR_InitializeVertexNormals (obj);
  return (obj);
}


void RemoveDirs (char *basestr, char *newstr)
/*
Used to remove any directories in front of the filename
/ and \ are treated as the same thing
Ex: /progs\blah\misc/a.html would return a.html
*/
{
PR_DWORD i;
PR_DWORD modify;

  modify = -1;
  for (i = strlen (basestr)-1; i >= 0; i--)
    {
     if ((basestr[i] == '/') || (basestr[i] == '\\'))
        {
         modify = i+1;
         break;
        }
    }

  if (modify == -1)
    {
     strcpy (newstr, basestr);
     return;
    }

  for (i = modify; i < strlen (basestr); i++)
     newstr[i-modify] = basestr[i];
  newstr[i-modify] = 0;
  return;
}




mdl_t *PR_LoadMDLFromPak (char *fname,char *pakname)
{
FILE        *f;
mdlheader_t mdlheader;
pakheader_t pakheader;
pakentry_t  pakentry;
PR_DWORD    numentries; 
char        cmpname[64];
PR_DWORD    i,j;
PR_DWORD    FOUND;
mdl_t   *mdl;


  if (pakname == NULL)
    {
     /* Library code */
     if  (wgtlibrary == NULL)
       {
        libf = fopen (fname, "rb");
        if (libf == NULL)
          return (NULL);
        f = libf;
       }
     else
       {
        if ((libf = fopen (wgtlibrary, "rb")) == NULL)
          return (NULL);
        readheader ();
        findfile (fname);
        if (lresult == 1)
          fseek (libf, lfpos, SEEK_SET);
        else
          {
           fclose (libf);
           return (NULL);
          }
        f = libf;
       }
    }
  else
    {
     f = fopen (pakname, "rb");
     if (f == NULL)
        PRGUI_Error ("Unable to open pakfile");
    
	
     fread (&pakheader, sizeof(pakheader_t), 1, f);
     numentries = pakheader.dirsize / (sizeof(pakentry_t));

     fseek (f, pakheader.diroffset, SEEK_SET);
     FOUND = 0;

     for (i = 0; i < numentries; i++)
       {
        fread (&pakentry, sizeof(pakentry_t), 1, f);
        if (strstr(pakentry.filename, ".mdl") != NULL)
          {
           RemoveDirs (pakentry.filename, cmpname);
           if (stricmp (fname, cmpname) == 0)
             {
              FOUND = 1;
              break;
             }
          }
       }
	
     if (!FOUND)
       {
        fclose (f);
        PRGUI_Error ("Couldnt find model in the pakfile");
       }
	
     fseek (f, pakentry.offset, SEEK_SET); 
    }

  mdl = (mdl_t *)malloc (sizeof (mdl_t)); 
  memset (mdl, 0, sizeof (mdl_t));        

  fread (&mdlheader, sizeof(mdlheader_t), 1, f);

  mdl->origin     = mdlheader.origin;
  mdl->scale      = mdlheader.scale;
  mdl->numverts   = mdlheader.numverts;
  mdl->numtris    = mdlheader.numtris;
  mdl->numskins   = mdlheader.numskins;
  mdl->numframes  = mdlheader.numframes;
  mdl->skinwidth  = mdlheader.skinwidth;
  mdl->skinheight = mdlheader.skinheight;

  /* Skin Loading */
  mdl->skins = (skin_t *) malloc (mdlheader.numskins * sizeof(skin_t)); 
  memset (mdl->skins, 0, mdlheader.numskins * sizeof(skin_t));

  for (i = 0;i < mdlheader.numskins; i++)
    {
     /* Get skin type (group or single) */
     fread (&(mdl->skins[i].type), sizeof(long), 1, f);   
     if (mdl->skins[i].type == 0)
       {
        /* Single skin */
        mdl->skins[i].numgroupskins = 1;
       
        /* Init the skin memory */
        mdl->skins[i].skin    = (unsigned char **)malloc (sizeof(unsigned char *)); 
        mdl->skins[i].skin[0] = (unsigned char *)malloc (mdlheader.skinwidth * mdlheader.skinheight);          
			
        /* read skin data */
        fread (mdl->skins[i].skin[0], mdlheader.skinwidth * mdlheader.skinheight, 1, f);

        /* init interval crap. dunno what its for */
        mdl->skins[i].interval = (float *)malloc (sizeof(float));             
        *(mdl->skins[i].interval) = 0.f;
       }
     else
       {
        /* Group skin */
        fread (&(mdl->skins[i].numgroupskins), sizeof(long), 1, f);

        /* load group skin interval data */
        mdl->skins[i].interval = (float *)malloc (sizeof(float) * (mdl->skins[i].numgroupskins));
        for (j = 0; j < mdl->skins[i].numgroupskins; j++)
           fread (&(mdl->skins[i].interval[j]), sizeof(float), 1, f);
			
        /* load group skin data */
        mdl->skins[i].skin = (unsigned char **)malloc (sizeof(unsigned char *) * (mdl->skins[i].numgroupskins));
						
        for (j = 0; j < mdl->skins[i].numgroupskins; j++)
          {
           mdl->skins[i].skin[j] = (unsigned char *)malloc (mdlheader.skinwidth*mdlheader.skinheight);
           fread (mdl->skins[i].skin[j], mdlheader.skinwidth * mdlheader.skinheight, 1, f);
          }
       }
    }


  /* Vertex loading (texture vertices, not spatial vertices) */
  mdl->verts = (stvert_t *)malloc (mdlheader.numverts * sizeof(stvert_t));
  fread (mdl->verts, mdlheader.numverts * sizeof(stvert_t), 1, f);

  /* Triangle loading */
  mdl->triangles = (itriangle_t *)malloc (mdlheader.numtris * sizeof(itriangle_t));
  fread (mdl->triangles, mdlheader.numtris * sizeof(itriangle_t), 1, f);

  /* Frame loading (spatial vertex information)	*/
  mdl->frames = (frame_t *)malloc (mdlheader.numframes * sizeof(frame_t)); 
  for (i = 0;i < mdlheader.numframes; i++)
    {
     /* Read the frame type (group or single) */
     fread (&(mdl->frames[i].type), sizeof(long), 1, f);
		
     if (mdl->frames[i].type == 0)
        {
         /* Single frame */               
         mdl->frames[i].numgroupframes = 1;

         /* Get frame info */
         mdl->frames[i].info = (frameinfo_t *)malloc (sizeof(frameinfo_t));            
         fread (mdl->frames[i].info, sizeof(frameinfo_t), 1, f);

         /* Get frame data */
         mdl->frames[i].data = (trivertx_t **)malloc (sizeof(unsigned char *));               
         mdl->frames[i].data[0] = (trivertx_t *)malloc (mdlheader.numverts * sizeof(trivertx_t));
         fread (mdl->frames[i].data[0], mdlheader.numverts * sizeof(trivertx_t), 1, f);

         /* make fake "interval" data */
         mdl->frames[i].interval = (float *)malloc (sizeof(float));            
         *(mdl->frames[i].interval) = 0.f;
        }
      else
        {
         /* Group frame */
         /* get group information */
         fread (&(mdl->frames[i].numgroupframes), sizeof(long), 1, f);
         fread (&(mdl->frames[i].groupmin), sizeof(trivertx_t), 1, f);
         fread (&(mdl->frames[i].groupmax), sizeof(trivertx_t), 1, f);
			
         /* init necessary data memory */
         mdl->frames[i].data = (trivertx_t **)malloc (sizeof(unsigned char *) * (mdl->frames[i].numgroupframes));
			
         /* init necessary info memory */
         mdl->frames[i].info = (frameinfo_t *)malloc(sizeof(frameinfo_t) * (mdl->frames[i].numgroupframes));
			
         /* load group frame interval data */
         mdl->frames[i].interval = (float *)malloc(sizeof(float) * (mdl->frames[i].numgroupframes));
			
         for (j = 0; j < mdl->frames[i].numgroupframes; j++)
           fread (&(mdl->frames[i].interval[j]), sizeof(float), 1, f);

         for (j = 0;j < mdl->frames[i].numgroupframes; j++)
           {
            /* read in the frame info */
            fread (&(mdl->frames[i].info[j]), sizeof(frameinfo_t), 1, f);

            /* allocate and read in the frame data */
            mdl->frames[i].data[j] = (trivertx_t *)malloc (mdlheader.numverts * sizeof(trivertx_t));                        
            fread (mdl->frames[i].data[j], mdlheader.numverts * sizeof(trivertx_t), 1, f);
           }
        }
    }


  /* Check if everything was read ok. We should be at the end of the
     pak entry right now */
  if (pakname != NULL)
    {
     if ((ftell (f) - pakentry.offset) != pakentry.size)
       {
        fclose (f);
        PRGUI_Error ("Error reading model from pakfile");
       }
    }
  fclose (f);
  return(mdl);
}




mdl_t2 *PR_LoadMD2FromPak (char *fname,char *pakname)
{
FILE        *f;
pakheader_t pakheader;
pakentry_t  pakentry;
PR_DWORD    numentries; 
PR_DWORD    i;
PR_DWORD    FOUND;
mdl_t2      *md2;
PR_DWORD    offset;

  if (pakname == NULL)
    {
     /* Library code */
     if  (wgtlibrary == NULL)
       {
        libf = fopen (fname, "rb");
        if (libf == NULL)
          return (NULL);
        f = libf;
       }
     else
       {
        if ((libf = fopen (wgtlibrary, "rb")) == NULL)
          return (NULL);
        readheader ();
        findfile (fname);
        if (lresult == 1)
          fseek (libf, lfpos, SEEK_SET);
        else
          {
           fclose (libf);
           return (NULL);
          }
        f = libf;
       }
    }
  else
    {
     f = fopen (pakname, "rb");
     if (f == NULL)
        PRGUI_Error ("Unable to open pakfile");
    
	
     fread (&pakheader, sizeof(pakheader_t), 1, f);
     numentries = pakheader.dirsize / (sizeof(pakentry_t));

     fseek (f, pakheader.diroffset, SEEK_SET);
     FOUND = 0;

     for (i = 0; i < numentries; i++)
       {
        fread (&pakentry, sizeof(pakentry_t), 1, f);
        if (strstr(pakentry.filename, ".md2") != NULL)
          {
//           RemoveDirs (pakentry.filename, cmpname);
//           printf ("%s\n", cmpname);
           if (stricmp (fname, pakentry.filename) == 0)
             {
              FOUND = 1;
              break;
             }
          }
       }
	
     if (!FOUND)
       {
        fclose (f);
        PRGUI_Error ("Couldnt find model in the pakfile");
       }
	
     fseek (f, pakentry.offset, SEEK_SET); 
    }

  offset = ftell (f);

  md2 = (mdl_t2 *)malloc (sizeof (mdl_t2)); 
  memset (md2, 0, sizeof (mdl_t2));

  fread (&md2header, sizeof(mdlheader_t2), 1, f);

  md2->numframes  = md2header.num_frames;
  md2->framesize   = md2header.framesize;
  md2->num_glcmds  = md2header.num_glcmds;  
  md2->skinwidth   = md2header.skinwidth;
  md2->skinheight  = md2header.skinheight;    

  fseek(f, offset + md2header.ofs_glcmds, SEEK_SET);

  md2->glcmds = (long *)malloc(md2->num_glcmds * sizeof(long));
  fread(md2->glcmds, md2->num_glcmds * sizeof(long), 1, f);

  fseek(f, offset + md2header.ofs_frames, SEEK_SET);
  md2->frames = (frameinfo_t2 *)malloc(md2->framesize * md2->numframes);
  fread(md2->frames, md2->framesize * md2->numframes, 1, f);

  fseek(f, offset + md2header.ofs_skins, SEEK_SET);
  md2->num_skins = md2header.num_skins;

  if (md2->num_skins > 0)
    {
     md2->skinnames = (char **)malloc(sizeof(char *) * md2->num_skins);
     for (i = 0; i < md2->num_skins; i++)
       {
        md2->skinnames[i] = (char *)malloc(64);
        fread(md2->skinnames[i], 64, 1, f);
       }
    }
  else
    {
     md2->skinnames = (char **)malloc(sizeof(char *) * 1);
     md2->skinnames[0] = (char *)malloc(64);
     strcpy (md2->skinnames[0], pcxfile);
     md2->num_skins = 1;
    }


  fclose (f);
  return(md2);
}



void PR_ImportMD2Textures (mdl_t2 *md2)
{
PR_MATERIAL *m;
PR_DWORD matnum;
block skinpic;
char texname[20];
unsigned char *bits;
PR_DWORD pitch;

PR_UCHAR cur_byte, run_len;
PR_WORD w, h;
int size, offset = 0;
PR_DWORD j, texnum;
FILE *f;

  PR_LoadTable ("quake2.tab");
  PR_SetTextureFormat (TEXTURE_NORMAL);

  for (matnum = 0; matnum < md2->num_skins; matnum++)
    {
     if (pakfile[0]==0)
       {
        f = fopen (md2->skinnames[0], "rb");
        if (f != NULL)
          {
           fseek (f, 0, SEEK_END);
           size   = ftell (f);
           fseek (f, 0, SEEK_SET);
          }
      }

     else
       f = FindInPak (pakfile, md2->skinnames[0], &size, &offset);

     if (f == NULL)
       PRGUI_Error ("Cannot open texture");

     fseek(f, offset + 8,SEEK_SET);
     fread(&w,sizeof(short),1,f);
     fread(&h,sizeof(short),1,f);
  
     w++;
     h++;  

     if ((w < 1) || (h < 1))
       PRGUI_Error ("Invalid Texture Size");

     /* Create the texture */
     skinpic = wallocblock (w, h);
 
     #if defined (MSDD) || defined (WTDD)
     bits = wgetblocksurface (skinpic, &pitch);
     if (bits == NULL)
        return;
     #else
     bits = skinpic + 4;
     pitch = w;
     #endif
  
     fseek(f, offset + 128,SEEK_SET);
     j = 0;

     while (h > 0)
       {
        j = 0;
        while (j < w)
          {
           cur_byte = fgetc(f);
           if((cur_byte & 0xC0) == 0xC0)
             {
              run_len = cur_byte & 0x3F;
              cur_byte = fgetc(f);
              for(;(run_len > 0) && (j < w*h); run_len--, j++)
                bits[j] = cur_byte;
             }
           else
             {
              bits[j] = cur_byte;
              j++;
             }
          }
        h--;
		bits += pitch;
       }


     //read the pcx palette
     fseek (f, offset + size-768,SEEK_SET);
     fread (global_palette,768,1,f);
     fclose (f);

     for (j = 0; j < 256; j++)
       {
        global_palette[j].r >>= 2;
        global_palette[j].g >>= 2;
        global_palette[j].b >>= 2;
       }

     #if defined (MSDD) || defined (WTDD)
        wunlocksurface (skinpic);
     #endif
     
     sprintf (texname, "skin%i.pcx", matnum);
     
     m = &PR_ObjectMaterialList[matnum];

     PR_SetMaterialName (m, texname);
     if (shade_mode == GOURAUD_MODE)
       PR_SetMaterialMethod (m, T_GOURAUD_TEXTURED8);
     else
       PR_SetMaterialMethod (m, T_TEXTURED);
     PR_SetMaterialBaseColor (m, 0);
     PR_SetMaterialColor (m, 255, 255, 255);
     texnum = PR_AddTexture (texname, skinpic);
     PR_SetMaterialTexture (m, texnum);
     PR_SetMaterialTable (m, 0);
     PR_SetMaterialShades (m, 31);
     PR_SetMaterialEnvironmentMap (m, 0);
     PR_SetMaterialMipMapState (m, FALSE);
     m->a = 255;


     PR_AddMaterial (m);

     md2->s_scale = PR_WorldTextures[texnum].xscale;
     md2->t_scale = PR_WorldTextures[texnum].yscale;
    }
}


typedef struct
{
PR_REAL u, v;
PR_DWORD vindex;
} gl_vtx;
gl_vtx gl_verts[64];    /* Temp storage for strips and fans */


void MakeFace (PR_FACE *faceptr, PR_SEGMENT *seg, PR_DWORD v1, PR_DWORD v2, PR_DWORD v3)
{
PR_FACE_DATA_REAL *datareal;
PR_FACE_DATA *datafix;

  faceptr->vertex1 = PR_GetVertexPointer (seg->vertex_list, gl_verts[v1].vindex);
  faceptr->vertex2 = PR_GetVertexPointer (seg->vertex_list, gl_verts[v2].vindex);
  faceptr->vertex3 = PR_GetVertexPointer (seg->vertex_list, gl_verts[v3].vindex);

  faceptr->material = 0;
  faceptr->backmaterial = 0;

  faceptr->flags = FFLAG_FRONT_VISIBLE; 


  if (PR_Settings.RealUV)
    {
     datareal = (PR_FACE_DATA_REAL *)&faceptr->face_data;

     datareal->u[0] = gl_verts[v1].u;
     datareal->v[0] = gl_verts[v1].v;
     datareal->col[0] = 0;

     datareal->u[1] = gl_verts[v2].u;
     datareal->v[1] = gl_verts[v2].v;
     datareal->col[1] = 0;

     datareal->u[2] = gl_verts[v3].u;
     datareal->v[2] = gl_verts[v3].v;
     datareal->col[2] = 0;
    }
  else
    {
     datafix = (PR_FACE_DATA *)&faceptr->face_data;

     datafix->u[0] = (gl_verts[v1].u * 65536.0f * md2header.skinwidth);
     datafix->v[0] = (gl_verts[v1].v * 65536.0f * md2header.skinheight);
     datafix->col[0] = 0;

     datafix->u[1] = (gl_verts[v2].u * 65536.0f * md2header.skinwidth);
     datafix->v[1] = (gl_verts[v2].v * 65536.0f * md2header.skinheight);
     datafix->col[1] = 0;

     datafix->u[2] = (gl_verts[v3].u * 65536.0f * md2header.skinwidth);
     datafix->v[2] = (gl_verts[v3].v * 65536.0f * md2header.skinheight);
     datafix->col[2] = 0;
    }
}



PR_OBJECT *PR_ImportMD2 (mdl_t2 *md2)
/* Converts an MD2 structure to a PR_OBJECT structure */
{
int         *command;
int         num_verts,vert_index;
int         command_type;

PR_OBJECT *obj;
PR_SEGMENT *seg;
PR_FACE *faceptr;
PR_DWORD i, frame, facenum;
frameinfo_t2 *frameptr;
PR_VERTEX *vptr;
PR_REAL s, t;

  PR_ImportMD2Textures (md2);
  wloadpalette ("quake2.pal", global_palette);

  /* Allocate the object */
  obj = PR_AllocObject ();

  /* Allocate the segment */
  obj->segment_list = PR_AllocSegment ();
  obj->num_segments = 1;
  obj->num_frames = -1;

  /* Allocate the faces */

  seg = obj->segment_list;
  seg->num_faces = md2header.num_tris;
  seg->face_list = malloc (seg->num_faces * sizeof (PR_FACE));
  memset (seg->face_list, 0, seg->num_faces * sizeof (PR_FACE));

  seg->num_vertices = (md2->framesize - 40) / 4;
  seg->vertex_list = malloc (seg->num_vertices * sizeof (PR_VERTEX));
  seg->vertex_data = malloc (seg->num_vertices * sizeof (PR_VERTEX_DATA));
  memset (seg->vertex_list, 0, seg->num_vertices * sizeof (PR_VERTEX));
  memset (seg->vertex_data, 0, seg->num_vertices * sizeof (PR_VERTEX_DATA));

  seg->flags = 0;

  /* Set up vertices, normals, and face data */

  for (frame = 0; frame < md2->numframes; frame++)
    {
     frame_list[frame] = malloc (seg->num_vertices * sizeof (PR_VERTEX));
     memset (frame_list[frame], 0, seg->num_vertices * sizeof (PR_VERTEX));

     frameptr = (frameinfo_t2 *)((unsigned char *)md2->frames + md2->framesize * frame);
     vptr = frame_list[frame];

     for (i = 0; i < seg->num_vertices; i++)
       {                                                              
        vptr[i].z = ((PR_REAL)frameptr->verts[i].x * frameptr->scale.x + frameptr->origin.x) * 50.0f;
        vptr[i].x = ((PR_REAL)frameptr->verts[i].y * frameptr->scale.y + frameptr->origin.y) * 50.0f;
        vptr[i].y = ((PR_REAL)frameptr->verts[i].z * frameptr->scale.z + frameptr->origin.z) * 50.0f;
        vptr[i].vdata = &seg->vertex_data[i];
       }
    }

   memcpy (seg->vertex_list, frame_list[0], seg->num_vertices * sizeof (PR_VERTEX));


   /* Create the faces */
   command = (int *)md2->glcmds;
   faceptr = seg->face_list;
   facenum = 0;

   while (*command)
     {
      if (*command > 0)
        {
         //triangle strip
         num_verts = *command;
         command_type = 0;
        }
      else
        {
         //triangle fan
         num_verts = -(*command);
         command_type = 1;
        }
    
      command++;
    
      for (i = 0; i < num_verts; i++)
        {
         //grab the floating point s and t
         s = *(float *)command;
         command++;
         t = *(float *)command;
         command++;

         vert_index = *command;
         command++;


         if (PR_Settings.RealUV)
           {
            if (PR_OutputDevice.devicetype == DEVICE_3DFX)
              {
               s *= md2->s_scale * 255.0f;
               t *= md2->t_scale * 255.0f;
              }
           }

         gl_verts[i].u = s;
         gl_verts[i].v = t;
         gl_verts[i].vindex = vert_index;
        }
     
    
      switch (command_type)
        {
         case 0:
          //tristrip
           for (i = 0;i < num_verts - 2; i++)
             {
              //this is to ensure correct vertex order
              if (i&1)
                MakeFace (faceptr, seg, i, i+2, i+1);
              else
                MakeFace (faceptr, seg, i, i+1, i+2);
              faceptr++;
              facenum++;
             }
           break;

         case 1:
         //trifan
           for (i = 0; i < num_verts - 2; i++)
             {
              MakeFace (faceptr, seg, 0, i+1, i+2);
              faceptr++;
              facenum++;
             }
           break;
         }
    }

  PR_ObjectCalcBoundingBox (obj);
  PR_FlipFaceNormals (obj);
  PR_InitializeFaceNormals (obj);
  PR_InitializeVertexNormals (obj);
  return (obj);
}


