/* Power Render GL Interface */

#ifndef _PRGL_
#define _PRGL_

#ifdef __cplusplus
  extern "C" {
#endif

extern HDC   PRGL_DC;
extern HGLRC PRGL_RC;


PR_DWORD PR_AddTextureGL (char *filename, block image);
/* -----------------------------------------------------------------------
   Function:   PR_AddTextureGL
   Purpose:    Adds a texture to the PR_WorldTextures array
   Parameters: filename - Filename of the texture (must be present)
               image - 8 bit texture image, or NULL if 3DF is used
   Return:     Texture number
   ----------------------------------------------------------------------- */

PR_DWORD PR_AddTextureWithAlphaGL (char *filename, block image, block alphaimage);
/* -----------------------------------------------------------------------
   Function:   PR_AddTextureWithAlphaGL
   Purpose:    Adds a texture to the PR_WorldTextures array
   Parameters: filename - Filename of the texture (must be present)
               image - 8 bit texture image
               alphaimage - 8 bit alpha image
   Return:     Texture number
   ----------------------------------------------------------------------- */


void PR_AllocTexturesGL (PR_DWORD maxtexture);
/* -----------------------------------------------------------------------
   Function:   PR_AllocTexturesGL
   Purpose:    Allocates room for a list of textures
   Parameters: maxtexture - Number of textures positions to allocate
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_BarGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1);
/* -----------------------------------------------------------------------
   Function:   PRGFX_BarGL
   Purpose:    Draws a bar (filled rectangle)
   Parameters: x0, y0, x1, y1 - Rectangle to fill with a solid color
   Return:     None
   ----------------------------------------------------------------------- */


void PR_BeginScreenGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_BeginScreenGL
   Purpose:    Initializes a buffer for direct access
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_ClearScreenGL (void);
/* -----------------------------------------------------------------------
   Function:   PRGFX_ClearScreenGL
   Purpose:    Clears the current screen 
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


void PR_ClearViewportGL (PR_VIEWPORT *viewport);
/* -----------------------------------------------------------------------
   Function:   PR_ClearViewportGL
   Purpose:    Clears color and z buffers of a viewport
   Parameters: viewport - Pointer to the viewport 
   Return:     None
   ----------------------------------------------------------------------- */


void PR_ClearViewportOptGL (PR_VIEWPORT *viewport, PR_DWORD flags);
/* -----------------------------------------------------------------------
   Function:   PR_ClearViewportOptGL
   Purpose:    Optionally clears the color and z buffers
   Parameters: viewport - Pointer to the viewport
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_ClipGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1);
/* -----------------------------------------------------------------------
   Function:   PRGFX_ClipGL
   Purpose:    Sets the clipping region in device coordinates
   Parameters: x0, y0, x1, y1 - Clipping rectangle
   Return:     None
   ----------------------------------------------------------------------- */


void PR_DeleteAllTexturesGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_DeleteAllTexturesGL
   Purpose:    Deletes all the textures from the PR_WorldTextures array
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


PR_DWORD PR_DetectGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_DetectGL
   Purpose:    Looks for the presence of 3D accelerated hardware 
   Parameters: None
   Return:     TRUE if found
   ----------------------------------------------------------------------- */


void PR_EndScreenGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_EndScreenGL
   Purpose:    Closes a screen after direct access
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


void PR_FlipGL (PR_DWORD retrace);
/* -----------------------------------------------------------------------
   Function:   PR_FlipGL
   Purpose:    Swaps front and back buffers
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_HLineGL (PR_DWORD x0, PR_DWORD x1, PR_DWORD y0);
/* -----------------------------------------------------------------------
   Function:   PRGFX_HLineGL
   Purpose:    Draws a horizontal line
   Parameters: x0, x1 - endpoints
               y0     - scan line
   Return:     None
   ----------------------------------------------------------------------- */


void PR_InitializeGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_InitializeGL
   Purpose:    Initializes the GL hardware routines
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_LineGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1);
/* -----------------------------------------------------------------------
   Function:   PRGFX_LineGL
   Purpose:    Draws a diagonal line
   Parameters: x0, y0, x1, y1 - Endpoints of line
   Return:     None
   ----------------------------------------------------------------------- */


PR_DWORD PR_LoadTextureGL (char *filename);
/* -----------------------------------------------------------------------
   Function:   PR_LoadTextureGL
   Purpose:    Loads the texture if it hasn't been already
   Parameters: filename - Filename of the texture to load
   Return:     Texture number
   ----------------------------------------------------------------------- */


PR_DWORD PR_LoadTextureWithAlphaGL (char *filename, char *alphafilename);
/* -----------------------------------------------------------------------
   Function:   PR_LoadTextureWithAlphaGL
   Purpose:    Loads the texture if it hasn't been already
   Parameters: filename - Filename of the texture to load
               alphafilename - Filename of the alpha texture
   Return:     Texture number
   ----------------------------------------------------------------------- */


PR_UDWORD PRGFX_MakeColorGL (PR_DWORD r, PR_DWORD g, PR_DWORD b);
/* -----------------------------------------------------------------------
   Function:   PRGFX_MakeColorGL
   Purpose:    Returns a color value for the RGB value given
   Parameters: r,g,b - Red, green, blue components (0-64)
   Return:     Packed color value (RGBA8888)
   ----------------------------------------------------------------------- */


block PRGFX_NewBlockGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1);
/* -----------------------------------------------------------------------
   Function:   PRGFX_NewBlockGL
   Purpose:    Grabs a section of the render buffer into memory
   Parameters: x0, y0, x1, y1 - Rectangular region to grab
   Return:     A block of memory in the format of the render buffer.
               (8 bit, RGB565 etc)
   ----------------------------------------------------------------------- */


void PR_OpenScreenGL (PR_DWORD screen);
/* -----------------------------------------------------------------------
   Function:   PR_OpenScreenGL
   Purpose:    Selects a buffer to render on 
   Parameters: screen - PR_FRONTBUFFER or PR_BACKBUFFER
   Return:     None
   ----------------------------------------------------------------------- */


PR_DWORD PRGFX_outcharGL (short asc, PR_DWORD xc, PR_DWORD yc, wgtfont font);
/* -----------------------------------------------------------------------
   Function:   PRGFX_outcharGL
   Purpose:    Displays a single character
   Parameters: asc - Ascii code
               xc - X coordinate
               yc - Y coordinate
               font - Pointer to font
   Return:     width of character
   ----------------------------------------------------------------------- */


void PRGFX_OutTextXYGL (PR_DWORD x, PR_DWORD y, wgtfont font, char *string);
/* -----------------------------------------------------------------------
   Function:   PRGFX_OutTextXYGL
   Purpose:    Draws an unformatted string
   Parameters: x, y - Screen location
               font - Font to use
               string - String to display
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_PutBlockGL (PR_DWORD x, PR_DWORD y,
                         PR_DWORD width, PR_DWORD height,
                         block buffer);
/* -----------------------------------------------------------------------
   Function:   PRGFX_PutBlockGL
   Purpose:    Pastes a block of conventional memory to video memory
   Parameters: x,y - Location on screen
               width - Width of source block
               height - Height of source block
               src - Block containing source data
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_PutBlock8bitGL (PR_DWORD x, PR_DWORD y, block src, color *pal,
                             PR_DWORD method);
/* -----------------------------------------------------------------------
   Function:   PRGFX_PutBlock8bitGL
   Purpose:    Pastes an image in 8 bit color indexed format to the screen
   Parameters: x,y - Location on screen
               src - Block containing source data
               pal - Palette used
               method - NORMAL or XRAY
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_PutTextureGL (PR_DWORD x, PR_DWORD y, PR_DWORD texnum, PR_DWORD mode);
/* -----------------------------------------------------------------------
   Function:   PRGFX_PutTextureGL
   Purpose:    Displays a texture at (x,y)
   Parameters: x, y - Location on screen
               texnum - Texture number
               mode - Display mode
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_RectangleGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1);
/* -----------------------------------------------------------------------
   Function:   PRGFX_RectangleGL
   Purpose:    Draws a rectangle (hollow rectangle)
   Parameters: x0, y0, x1, y1 - Rectangle coordinates
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_ResizeTextureGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD x1, PR_DWORD y1,
                           PR_DWORD texnum, PR_DWORD mode);
/* -----------------------------------------------------------------------
   Function:   PRGFX_ResizeTextureGL
   Purpose:    Resizes the texture into the specified area on the screen 
   Parameters: x0, y0, x1, y1 - Rectangular region
               texnum - Texture number
               mode - Display mode
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_SetColorGL (PR_DWORD col);
/* -----------------------------------------------------------------------
   Function:   PRGFX_SetColorGL
   Purpose:    Sets the current drawing color
   Parameters: col - Color index or packed value
   Return:     None
   ----------------------------------------------------------------------- */


PR_DWORD PR_SetModeGL (PR_DWORD width, PR_DWORD height, PR_DWORD refrate);
/* -----------------------------------------------------------------------
   Function:   PR_SetModeGL
   Purpose:    Attempts to initialize a video mode 
   Parameters: width - Width of screen
               height - Height of screen
               refrate - Refresh rate
   Return:     MODE_INVALID, MODE_OPEN_ERROR, MODE_SUCCESS
   ----------------------------------------------------------------------- */


void PR_ShutdownGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_ShutdownGL
   Purpose:    Shuts down a display device 
   Parameters: None
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_VLineGL (PR_DWORD x0, PR_DWORD y0, PR_DWORD y1);
/* -----------------------------------------------------------------------
   Function:   PRGFX_VLineGL
   Purpose:    Draws a vertical line
   Parameters: x0 - X Coordinate
               y0, y1 - Endpoints
   Return:     None
   ----------------------------------------------------------------------- */



void PR_RenderFrameGL (void);
/* Renders all faces in the rendering list to the viewport */

void PR_ComputeVertexLightingGL (void);

void PR_AddElementGL (PR_FACE *elemptr, PR_VERTEX *pointlist,
                        PR_VERTEX_DATA *pointdata, PR_DWORD count);
/* Adds a face to the rendering list if it is visible */

void PR_SetFogColorGL (PR_DWORD col);

void PR_SetFogStateGL (PR_DWORD state);
/* Sets the global fog rendering state (0 or 1) */

void PR_SetFogRangeGL (PR_REAL near_dist, PR_REAL far_dist);
/* Sets the range of fog */

void PR_InitializeRenderMethods (void);
/* Sets the initial rendering methods array */

#define RENDERFLAG_PRESORT 1
extern PR_DWORD PR_MethodFlags[256];
void PR_RegisterRenderMethod (int method_number, void (*MethodProc)(),
                PR_DWORD flags);
/* Sets the routine used for initializing the state data for a particular
   rendering method. This is used to add new rendering methods to the
   ones existing in Power Render.  */

/* Texture caching functions */
void PR_SelectTextureGL (PR_DWORD texnum);
/* -----------------------------------------------------------------------
   Function:   PR_SelectTextureGL
   Purpose:    Prepares the hardware for using a texture.  Other textures
               may be swapped out from this routine.
   Parameters: texnum - Texture number to select
   Return:     None
   ----------------------------------------------------------------------- */


PR_UDWORD PR_GetTextureMemoryGL (void);
/* -----------------------------------------------------------------------
   Function:   PR_GetTextureMemoryGL
   Purpose:    Returns how much texture memory the 3D card has.
   Parameters: None
   Return:     Amount of texture memory
   ----------------------------------------------------------------------- */


PR_DWORD PR_DownloadTextureGL (PR_DWORD texnum, PR_DWORD force);
/* -----------------------------------------------------------------------
   Function:   PR_DownloadTextureGL
   Purpose:    Loads the texture into the video memory.
               If force is true, it may release other textures in order
               to make room.
   Parameters: texnum - Texture number
               force - TRUE or FALSE
   Return:     TEXTURE_LOADED or TEXTURE_RAM_FULL
   ----------------------------------------------------------------------- */


void PR_ReleaseTextureGL (PR_DWORD texnum);
/* -----------------------------------------------------------------------
   Function:   PR_ReleaseTextureGL
   Purpose:    Releases a texture in video memory so another texture can be
               swapped in.
   Parameters: texnum - Texture number to release
   Return:     None
   ----------------------------------------------------------------------- */


/* Added June 1/98 */
PR_DWORD PR_CalcVertexLightingGL (PR_VERTEX *vert, PR_VERTEX_DATA *vertdata);
/* -----------------------------------------------------------------------
   Function:   PR_CalcVertexLighting
   Purpose:    Calculates the lighting value for a given vertex 
   Parameters: vert - Pointer to the vertex
               vertdata - Pointer to the vertex data
   Return:     0 (result is stored in ADDPRIV_r, g, b)
   ----------------------------------------------------------------------- */

void PR_UpdateTextureGL (PR_DWORD texnum, PR_DWORD updatepal);
/* -----------------------------------------------------------------------
   Function:   PR_UpdateTextureGL
   Purpose:    Resends a texture to the 3D card, after the texture has
               been modified in some way.  
   Parameters: texnum - Texture number to update
               updatepal - True if the palette has changed
   Return:     None
   ----------------------------------------------------------------------- */


void PRGFX_LoadTriangleGL  (PR_VERTEX_DATA *a,
                            PR_VERTEX_DATA *b,
                            PR_VERTEX_DATA *c);

#ifdef __cplusplus
}
#endif

#endif

