#include <conio.h>
#include <stdlib.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>


/* Power Render Example 2

   Description:  Provides a minimal application which loads and rotate
                 a PRO file, in any video mode.
*/

                                             
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *entity;                     /* A single entity of the shape */
PR_DWORD animation_count;

void animation_timer (void)
{
 animation_count++;
}


void wgt_log_message (char *msg, ... )
{
  FILE *                outfile;
  va_list               arglist;

  outfile = fopen ("gl.log", "at");
  va_start (arglist, msg);
  vfprintf (outfile, msg, arglist);
  va_end (arglist);
  fprintf (outfile, "\n");
  fclose (outfile);
}


void main (int argc, char *argv[])
{
PR_DWORD vwidth, vheight;
PR_REAL cenx, ceny, cenz;
PR_UDWORD BLACK;                       /* Color index/value for black */
PR_DWORD move_count;


 wgt_log_message ("1");  
  ChooseDevice ();
 wgt_log_message ("2");  

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */
 wgt_log_message ("3");  
  InitializeDevices ();
 wgt_log_message ("4");  

  PR_AllocMaterials (32);              /* Allocate the materials, */
 wgt_log_message ("5");  
  PR_AllocTextures (32);               /* textures, */
 wgt_log_message ("6");  
  PR_AllocShadeTables (2);             /* and shade tables */
 wgt_log_message ("7");  

  /* Try 640x480 first */
  vwidth = 640;
  vheight = 480;
  if (PR_SetMode (vwidth, vheight, 60) != MODE_SUCCESS)
    {
 wgt_log_message ("8");  
     /* If that doesn't work, try the first one in the mode list */
     vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
     vheight = PR_VideoModes[0].height;
     PR_SetMode (vwidth, vheight, 60);
    }
 wgt_log_message ("9");  

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
 wgt_log_message ("10");  
  PR_SetViewport (&viewport);          /* Open a full screen viewport */
 wgt_log_message ("11");  

  /* Load the object */
  PR_SetTexturePath ("..\\data\\");
 wgt_log_message ("12");  
  object_shape = PR_LoadPRO ("..\\data\\cube.pro", LOAD_NORMAL);
 wgt_log_message ("13");  
  if (object_shape == NULL)
    PR_FatalError ("Cannot open data file", "ex02");
 wgt_log_message ("14");  
 
  PR_CenterObject (object_shape, &cenx, &ceny, &cenz);
 wgt_log_message ("15");  

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  entity = PR_CreateEntity (object_shape, "TEST ENTITY");

 wgt_log_message ("16");  
  camera = PR_AllocCamera ();          /* Initialize the camera */
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);
 wgt_log_message ("17");  

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);
 wgt_log_message ("18");  

  installkbd ();
  animation_count = 0;
  winittimer ();
  wstarttimer (animation_timer, TICKS (60));
 wgt_log_message ("19");  

  gui_font = NULL;
  PRGFX_SetTextForeground (PRGFX_MakeColor (63, 63, 63));
  PRGFX_SetTextBackground (BLACK);
  PRGFX_SetTextTransparent (TEXTFG);
 wgt_log_message ("20");  

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
 wgt_log_message ("21");  
     #ifdef WIN32
       UpdateMessages ();
     #endif      

 wgt_log_message ("22");  
     #if defined (MSDD) || defined (WTDD) || defined (WTGL) || defined (MSGL) 
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif
 wgt_log_message ("23");  

     PR_OpenScreen (PR_BACKBUFFER);
 wgt_log_message ("24");  

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */
 wgt_log_message ("25");  

     PR_NewFrame ();                   /* Begin a new frame */
 wgt_log_message ("26");  

     move_count = animation_count;
     animation_count = 0;

     PR_RotateEntity (entity, 1.5f, 1.2f, 0.7f);
                                               /* 1.5f * move_count,
                              1.2f * move_count,
                              0.7f * move_count);  */
 wgt_log_message ("27");  

     PR_TransformEntity (entity);      /* Transform the entity */
 wgt_log_message ("28");  
     PR_RenderEntity (entity);         /* Render the entity */
 wgt_log_message ("29");  
     PR_RenderFrame ();                /* Draw the scene */
 wgt_log_message ("30");  

     PR_Flip (1);                      /* Show the background page */
 wgt_log_message ("31");  
    }

  uninstallkbd ();
  wstoptimer ();
  wdonetimer ();
 wgt_log_message ("32");  

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (entity);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

